(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    516333,       9990]
NotebookOptionsPosition[    510804,       9805]
NotebookOutlinePosition[    511241,       9822]
CellTagsIndexPosition[    511198,       9819]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Test of Model Predictions", "Title"],

Cell["\<\
This Notebook shows how REAP can be used to test how the predictions of a \
given model change during the RG evolution to low energy.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$VersionNumber", "<", "6"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Get", "[", "\"\<Graphics`FilledPlot`\>\"", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Get", "[", "\"\<Graphics`Colors`\>\"", "]"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ForestGreen", "=", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.1", ",", "0.75", ",", "0.2"}], "]"}]}], ";"}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.435587848557613*^9, 3.435587856517888*^9}, {
  3.435655531948548*^9, 3.435655544087432*^9}, {3.635817659548285*^9, 
  3.6358176642028017`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Needs", "[", "\"\<REAP`RGEMSSM`\>\"", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<REAP`RGEPlotUtilities`\>\"", "]"}], 
  ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData["\<\"Loading REAP 1.10\\n\"\>"], "Print",
 CellChangeTimes->{3.636847119370451*^9, 3.636847428709425*^9, 
  3.6368474623570967`*^9}],

Cell[BoxData["\<\"SM0N: Two loop RG equation of \[Kappa] not implemented yet.\
\"\>"], "Print",
 CellChangeTimes->{3.636847119370451*^9, 3.636847428709425*^9, 
  3.636847462386983*^9}],

Cell[BoxData["\<\"SM: Two loop RG equations of Y\[Nu], M\[Nu]r and \[Kappa] \
not implemented yet.\"\>"], "Print",
 CellChangeTimes->{3.636847119370451*^9, 3.636847428709425*^9, 
  3.636847462412479*^9}]
}, Open  ]]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"MZ", "=", "91.12"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"GUT", "=", 
   RowBox[{"2", " ", 
    RowBox[{"10", "^", "16"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6368469804624157`*^9, 3.63684698357728*^9}}],

Cell["\<\
RGEReset is used in order to set all variables of RGESolver to their default \
values, if the notebook is executed several times.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGEReset", "[", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Definition of the model", "Section"],

Cell["\<\
We assume a supersymmetric model.  Below the SUSY-breaking scale (taken to be \
1 TeV), the SM is used as an effective theory.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"RGEReset", "[", "]"}]], "Input",
 CellChangeTimes->{{3.635818682949304*^9, 3.63581868546563*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"RGEAdd", "[", 
   RowBox[{"\"\<MSSM\>\"", ",", 
    RowBox[{"RGEtan\[Beta]", "\[Rule]", "50"}], ",", 
    RowBox[{"RGESearchTransitions", "\[Rule]", "True"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"RGEAdd", "[", 
   RowBox[{"\"\<SM\>\"", ",", 
    RowBox[{"RGECutoff", "\[Rule]", "1000"}]}], "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6358186882106533`*^9, 3.635818699794078*^9}}],

Cell[TextData[{
 "All couplings are given explicitly and no default values of the REAP \
package are used. The Yukawa couplings are given in \"RL convention\".  The \
neutrino masses are determined by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Y", "\[Nu]"], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ".\n\[Kappa] is set to 0. In a type II see-saw scenario, \[Kappa] is \
different from 0 below the mass threshold of the Higgs triplet."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGESetInitial", "[", 
   RowBox[{"GUT", ",", "\[IndentingNewLine]", 
    RowBox[{"RGEg1", "\[Rule]", "0.7044"}], ",", "\[IndentingNewLine]", 
    RowBox[{"RGEg2", "\[Rule]", "0.6965"}], ",", "\[IndentingNewLine]", 
    RowBox[{"RGEg3", "\[Rule]", "0.6984"}], ",", "\[IndentingNewLine]", 
    RowBox[{"RGEYu", "\[Rule]", 
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {
         RowBox[{"5.980", " ", 
          SuperscriptBox["10", 
           RowBox[{"-", "6"}]]}], "0", "0"},
        {"0", "0.001736", "0"},
        {"0", "0", "0.7417"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}],
     ",", "\[IndentingNewLine]", 
    RowBox[{"RGEYd", "\[Rule]", 
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {"0.0007314", 
         RowBox[{"0.001558", "\[InvisibleSpace]", "+", 
          RowBox[{"0.000022", " ", "\[ImaginaryI]"}]}], 
         RowBox[{"0.0002393", "\[InvisibleSpace]", "+", 
          RowBox[{"0.0006778", " ", "\[ImaginaryI]"}]}]},
        {
         RowBox[{"0.001555", "\[InvisibleSpace]", "-", 
          RowBox[{"0.000022", "\[ImaginaryI]"}]}], 
         RowBox[{"0.007555", "\[InvisibleSpace]"}], 
         RowBox[{"0.008913", "\[InvisibleSpace]", "-", 
          RowBox[{"0.000004", " ", "\[ImaginaryI]"}]}]},
        {
         RowBox[{"0.0002393", "\[InvisibleSpace]", "-", 
          RowBox[{"0.0006778", " ", "\[ImaginaryI]"}]}], 
         RowBox[{"0.008913", "\[InvisibleSpace]", "+", 
          RowBox[{"0.000004", " ", "\[ImaginaryI]"}]}], "0.2872"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}],
     ",", "\[IndentingNewLine]", 
    RowBox[{"RGEYe", "\[Rule]", 
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {"0.00009344", "0", "0"},
        {"0", "0.01972", "0"},
        {"0", "0", "0.33678"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}],
     ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{"RGEY\[Nu]", "\[Rule]", 
     RowBox[{"0.5", " ", 
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0.01", "0", "0"},
         {
          RowBox[{"0.0001", "\[ImaginaryI]"}], "0.1", "0"},
         {"0", "0", "1"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], "\[NoBreak]", 
       ")"}]}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"RGE\[Kappa]", "\[Rule]", 
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {"0", "0", "0"},
        {"0", "0", "0"},
        {"0", "0", "0"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}],
     ",", "\[IndentingNewLine]", 
    RowBox[{"RGEM\[Nu]r", "\[Rule]", 
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {
         RowBox[{
          RowBox[{"-", "3.660"}], " ", 
          SuperscriptBox["10", "10"]}], 
         RowBox[{"1.675", " ", 
          SuperscriptBox["10", "10"]}], 
         RowBox[{
          RowBox[{"-", "1.675"}], " ", 
          SuperscriptBox["10", "11"]}]},
        {
         RowBox[{"1.675", " ", 
          SuperscriptBox["10", "10"]}], 
         RowBox[{
          RowBox[{"-", "2.218"}], " ", 
          SuperscriptBox["10", "12"]}], 
         RowBox[{"1.098", " ", 
          SuperscriptBox["10", "13"]}]},
        {
         RowBox[{
          RowBox[{"-", "1.675"}], " ", 
          SuperscriptBox["10", "11"]}], 
         RowBox[{"1.098", " ", 
          SuperscriptBox["10", "13"]}], 
         RowBox[{
          RowBox[{"-", "2.218"}], " ", 
          SuperscriptBox["10", "14"]}]}
       },
       GridBoxAlignment->{
        "Columns" -> {{Decimal}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], "\[NoBreak]", 
      ")"}]}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGESolve", "[", 
   RowBox[{"MZ", ",", "GUT"}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plots", "Section"],

Cell[CellGroupData[{

Cell["Definitions", "Subsection"],

Cell["Define the plot region", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"tmin", ",", "tmax"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmin", "=", 
   RowBox[{"Log", "[", 
    RowBox[{"10", ",", "MZ"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tmax", "=", 
   RowBox[{"Log", "[", 
    RowBox[{"10", ",", "GUT"}], "]"}]}], ";"}]}], "Input"],

Cell["Define the functions that should be plotted", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"Mnu", ",", "Ye"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mnu", "[", "mu_", "]"}], ":=", 
  RowBox[{
   RowBox[{"RGEGetSolution", "[", 
    RowBox[{"mu", ",", "RGEM\[Nu]"}], "]"}], "//", 
   "Chop"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Ye", "[", "mu_", "]"}], ":=", 
   RowBox[{
    RowBox[{"RGEGetSolution", "[", 
     RowBox[{"mu", ",", "RGEYe"}], "]"}], "//", "Chop"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"ClearAll", "[", 
  RowBox[{
  "MixingPars", ",", "MixingParsLogScale", ",", "NuMasses", ",", 
   "NuMassesLogScale"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MixingPars", "[", "mu_", "]"}], ":=", 
  RowBox[{
   RowBox[{"MNSParameters", "[", 
    RowBox[{
     RowBox[{"Mnu", "[", "mu", "]"}], ",", 
     RowBox[{"Ye", "[", "mu", "]"}]}], "]"}], "\[LeftDoubleBracket]", "1", 
   "\[RightDoubleBracket]"}]}], "\n", 
 RowBox[{
  RowBox[{"MixingParsLogScale", "[", "x_", "]"}], ":=", 
  RowBox[{"MixingPars", "[", 
   RowBox[{"10", "^", "x"}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{"NuMasses", "[", "mu_", "]"}], ":=", 
  RowBox[{
   RowBox[{"MNSParameters", "[", 
    RowBox[{
     RowBox[{"Mnu", "[", "mu", "]"}], ",", 
     RowBox[{"Ye", "[", "mu", "]"}]}], "]"}], "\[LeftDoubleBracket]", "2", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"NuMassesLogScale", "[", "x_", "]"}], ":=", 
   RowBox[{"NuMasses", "[", 
    RowBox[{"10", "^", "x"}], "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"ClearAll", "[", 
  RowBox[{
  "\[Theta]12", ",", "\[Theta]13", ",", "\[Theta]23", ",", "\[Delta]", ",", 
   "\[CurlyPhi]1", ",", "\[CurlyPhi]2", ",", "m1", ",", "m2", ",", "m3", ",", 
   "\[CapitalDelta]m2sol", ",", "\[CapitalDelta]m2atm"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Theta]12", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
    "1", "\[RightDoubleBracket]"}], "/", "Degree"}]}], "\n", 
 RowBox[{
  RowBox[{"\[Theta]13", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
    "2", "\[RightDoubleBracket]"}], "/", "Degree"}]}], "\n", 
 RowBox[{
  RowBox[{"\[Theta]23", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
    "3", "\[RightDoubleBracket]"}], "/", 
   "Degree"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Delta]", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
    "4", "\[RightDoubleBracket]"}], "/", "Degree"}]}], "\n", 
 RowBox[{
  RowBox[{"\[CurlyPhi]1", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
    "8", "\[RightDoubleBracket]"}], "/", 
   "Degree"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CurlyPhi]2", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"MixingParsLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", 
    "9", "\[RightDoubleBracket]"}], "/", 
   "Degree"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m1", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{"NuMassesLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", "1", 
   "\[RightDoubleBracket]"}]}], "\n", 
 RowBox[{
  RowBox[{"m2", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{"NuMassesLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", "2", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m3", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{"NuMassesLogScale", "[", "t", "]"}], "\[LeftDoubleBracket]", "3", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalDelta]m2sol", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"m2", "[", "t", "]"}], "^", "2"}], "-", 
   RowBox[{
    RowBox[{"m1", "[", "t", "]"}], "^", "2"}]}]}], "\n", 
 RowBox[{
  RowBox[{"\[CapitalDelta]m2atm", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"m3", "[", "t", "]"}], "^", "2"}], "-", 
   RowBox[{
    RowBox[{"m2", "[", "t", "]"}], "^", 
    "2"}]}]}], "\[IndentingNewLine]"}], "Input"],

Cell["\<\
Test for compatibility with the experimental data
(allowed regions at the 3\[Sigma] CL as given in Maltoni et al., \
hep-ph/0405172v4)\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"CompareToExperiment", ",", "IsCompatibleWithExp"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CompareToExperiment", "[", "t_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"lNotComp", ",", "lComp"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"lComp", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lNotComp", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"\[Theta]12", "[", "t", "]"}], " ", "Degree"}], "]"}], 
           "^", "2"}], "\[GreaterEqual]", "0.23"}], "&&", 
         RowBox[{
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"\[Theta]12", "[", "t", "]"}], " ", "Degree"}], "]"}], 
           "^", "2"}], "\[LessEqual]", "0.38"}]}], ",", 
        RowBox[{"lComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lComp", ",", "\"\<\[Theta]12\>\""}], "]"}]}], ",", 
        RowBox[{"lNotComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lNotComp", ",", "\"\<\[Theta]12\>\""}], "]"}]}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sin", "[", 
           RowBox[{
            RowBox[{"\[Theta]13", "[", "t", "]"}], " ", "Degree"}], "]"}], 
          "^", "2"}], "\[LessEqual]", "0.051"}], ",", 
        RowBox[{"lComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lComp", ",", "\"\<\[Theta]13\>\""}], "]"}]}], ",", 
        RowBox[{"lNotComp", "[", 
         RowBox[{"lNotComp", ",", "\"\<\[Theta]13\>\""}], "]"}]}], "]"}], ";",
       "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"\[Theta]23", "[", "t", "]"}], " ", "Degree"}], "]"}], 
           "^", "2"}], "\[GreaterEqual]", "0.34"}], "&&", 
         RowBox[{
          RowBox[{
           RowBox[{"Sin", "[", 
            RowBox[{
             RowBox[{"\[Theta]23", "[", "t", "]"}], " ", "Degree"}], "]"}], 
           "^", "2"}], "\[LessEqual]", "0.68"}]}], ",", 
        RowBox[{"lComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lComp", ",", "\"\<\[Theta]23\>\""}], "]"}]}], ",", 
        RowBox[{"lNotComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lNotComp", ",", "\"\<\[Theta]23\>\""}], "]"}]}]}], "]"}], 
      ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[CapitalDelta]m2atm", "[", "t", "]"}], "]"}], 
          "\[GreaterEqual]", 
          RowBox[{"1.3", " ", 
           RowBox[{"10", "^", 
            RowBox[{"-", "3"}]}]}]}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"\[CapitalDelta]m2atm", "[", "t", "]"}], "]"}], 
          "\[LessEqual]", 
          RowBox[{"3.2", " ", 
           RowBox[{"10", "^", 
            RowBox[{"-", "3"}]}]}]}]}], ",", 
        RowBox[{"lComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lComp", ",", "\"\<\[CapitalDelta]m2atm\>\""}], "]"}]}], 
        ",", 
        RowBox[{"lNotComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lNotComp", ",", "\"\<\[CapitalDelta]m2atm\>\""}], 
          "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalDelta]m2sol", "[", "t", "]"}], "\[GreaterEqual]", 
          
          RowBox[{"7.1", " ", 
           RowBox[{"10", "^", 
            RowBox[{"-", "5"}]}]}]}], "&&", 
         RowBox[{
          RowBox[{"\[CapitalDelta]m2sol", "[", "t", "]"}], "\[LessEqual]", 
          RowBox[{"8.9", " ", 
           RowBox[{"10", "^", 
            RowBox[{"-", "5"}]}]}]}]}], ",", 
        RowBox[{"lComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lComp", ",", "\"\<\[CapitalDelta]m2sol\>\""}], "]"}]}], 
        ",", 
        RowBox[{"lNotComp", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"lNotComp", ",", "\"\<\[CapitalDelta]m2sol\>\""}], 
          "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The following parameters are compatible with the experimental \
data at the 3\[Sigma] CL:\>\"", ",", "lComp"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "lNotComp", "]"}], "\[Equal]", "0"}], ",", 
        RowBox[{"Return", "[", "1", "]"}], ",", 
        RowBox[{"Return", "[", "0", "]"}]}], "]"}], ";"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\n", 
 RowBox[{"IsCompatibleWithExp", ":=", 
  RowBox[{"CompareToExperiment", "[", 
   RowBox[{"Log", "[", 
    RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}]}], "Input"],

Cell["Positions of the thresholds", "Text"],

Cell[BoxData[{
 RowBox[{"ClearAll", "[", 
  RowBox[{"M1", ",", "M2", ",", "M3", ",", "ShadowEFT"}], "]"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"M1", ",", "M2", ",", "M3"}], "}"}], "=", 
   RowBox[{
    RowBox[{"RGEGetTransitions", "[", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"4", ",", "3", ",", "2"}], "}"}], ",", "1"}], 
    "\[RightDoubleBracket]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ShadowEFT", "=", 
   RowBox[{"RGEShadowEFT", "[", 
    RowBox[{"M1", ",", "M2", ",", "M3", ",", 
     SuperscriptBox["10", "17"]}], "]"}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compatibility to the experimental data", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IsCompatibleWithExp", ";"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The following parameters are compatible with the experimental \
data at the 3\[Sigma] CL:\"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"\<\"\[Theta]12\"\>", ",", "\<\"\[Theta]13\"\>", 
     ",", "\<\"\[Theta]23\"\>", ",", "\<\"\[CapitalDelta]m2atm\"\>", 
     ",", "\<\"\[CapitalDelta]m2sol\"\>"}], "}"}]}],
  SequenceForm[
  "The following parameters are compatible with the experimental data at the \
3\[Sigma] CL:", {
   "\[Theta]12", "\[Theta]13", "\[Theta]23", "\[CapitalDelta]m2atm", 
    "\[CapitalDelta]m2sol"}],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119701901*^9, 3.636847462731737*^9}]
}, Open  ]],

Cell["The low energy parameters are", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=\>\"", ",", 
    RowBox[{"\[Theta]12", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=\>\"", ",", 
    RowBox[{"\[Theta]13", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=\>\"", ",", 
    RowBox[{"\[Theta]23", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \(atm\
\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=\>\"", ",", 
    RowBox[{"\[CapitalDelta]m2atm", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", 
    "\"\<\!\(\*SuperscriptBox[\(eV\), \(2\)]\)\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \(sol\
\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=\>\"", ",", 
    RowBox[{"\[CapitalDelta]m2sol", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", 
    "\"\<\!\(\*SuperscriptBox[\(eV\), \(2\)]\)\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[\(m\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=\>\"", ",", 
    RowBox[{"m1", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", "\"\<eV\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[\(m\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=\>\"", ",", 
    RowBox[{"m2", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", "\"\<eV\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<\!\(\*SubscriptBox[\(m\), \(3\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=\>\"", ",", 
    RowBox[{"m3", "[", 
     RowBox[{"Log", "[", 
      RowBox[{"10", ",", "MZ"}], "]"}], "]"}], ",", "\"\<eV\>\""}], "]"}], 
  ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(12\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "32.49476897307687`", 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=", 32.49476897307687, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462760044*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(13\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "0.10085308143146259`", 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=", 0.10085308143146259`, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462765916*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(23\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "46.96199065440152`", 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\))=", 46.96199065440152, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.63684746276731*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[SuperscriptBox[\\(\[CapitalDelta]m\\), \
\\(2\\)], \\(atm\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "0.00209299580876404`", 
   "\[InvisibleSpace]", "\<\"\\!\\(\\*SuperscriptBox[\\(eV\\), \
\\(2\\)]\\)\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \
\(atm\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=", 0.00209299580876404, 
   "\!\(\*SuperscriptBox[\(eV\), \(2\)]\)"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462768564*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[SuperscriptBox[\\(\[CapitalDelta]m\\), \
\\(2\\)], \\(sol\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "0.00007720436037464298`", 
   "\[InvisibleSpace]", "\<\"\\!\\(\\*SuperscriptBox[\\(eV\\), \
\\(2\\)]\\)\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \
\(sol\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=", 0.00007720436037464298, 
   "\!\(\*SuperscriptBox[\(eV\), \(2\)]\)"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462769868*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[\\(m\\), \
\\(1\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "0.014843437266616178`", 
   "\[InvisibleSpace]", "\<\"eV\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[\(m\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=", 
   0.014843437266616178`, "eV"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462771184*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[\\(m\\), \
\\(2\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "0.017249115637116383`", 
   "\[InvisibleSpace]", "\<\"eV\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[\(m\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=", 
   0.017249115637116383`, "eV"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462772422*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"\\!\\(\\*SubscriptBox[\\(m\\), \
\\(3\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\))=\"\>", 
   "\[InvisibleSpace]", "0.04889302403233669`", 
   "\[InvisibleSpace]", "\<\"eV\"\>"}],
  SequenceForm[
  "\!\(\*SubscriptBox[\(m\), \(3\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\))=", 
   0.04889302403233669, "eV"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847119716612*^9, 3.636847462773744*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mixing Angles", "Subsection"],

Cell["Draw the plot for the mixing angles", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Theta]12", "[", "t", "]"}], ",", 
       RowBox[{"\[Theta]13", "[", "t", "]"}], ",", 
       RowBox[{"\[Theta]23", "[", "t", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\>\""}], "}"}]}], ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "15"}], ",", 
            RowBox[{
             RowBox[{"ToString", "[", 
              RowBox[{"i", "*", "15"}], "]"}], "<>", "\"\<\[Degree]\>\""}]}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "3"}], "}"}]}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "15"}], ",", "\"\<\>\""}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "3"}], "}"}]}], "]"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", "ForestGreen", ",", "Blue"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["\[Theta]", "12"], ",", 
            RowBox[{"FontColor", "\[Rule]", "Red"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"\[Theta]12", "[", "tmin", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["\[Theta]", "13"], ",", 
            RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"\[Theta]13", "[", "tmin", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", 
            RowBox[{"-", "1"}]}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
        
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["\[Theta]", "23"], ",", 
            RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"\[Theta]23", "[", "tmin", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}]}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.435655087000834*^9, 3.435655149172597*^9}, {
   3.435655291841278*^9, 3.435655355632481*^9}, {3.435655474900189*^9, 
   3.43565552545153*^9}, 3.435655622860778*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[1, 0, 0], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwd1Hk8VOsDx3HLmBmUJUQRkiKOTNeaOTPzJbexlLUSSVqpJJfQtUuUn1JZ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     "]]}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k4VO0bB3AJMyhUaLOFsk0zp7IUcd+iVCpUL9HykupVSVGWhLKmSIuK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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k41NsfB3DEzEhlxBTJjC3LzBdfN9uV5ZPc71f8lKWrRmhuKDJFihaD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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 0},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[Theta]", "12"], FontColor -> RGBColor[1, 0, 0], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[1, 0, 0], FontSize -> 12]& ], 
      TraditionalForm], {1.9596137110710439`, 32.49476897307687}, {-0.8, 
     1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[Theta]", "13"], FontColor -> RGBColor[0.1, 0.75, 0.2],
         FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {1.9596137110710439`, 
     0.10085308143146259`}, {-0.8, -1}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[Theta]", "23"], FontColor -> RGBColor[0, 0, 1], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {1.9596137110710439`, 46.96199065440152}, {-0.8, 
     1.4}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox["\"\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{0, 
       FormBox["\"0\[Degree]\"", TraditionalForm]}, {15, 
       FormBox["\"15\[Degree]\"", TraditionalForm]}, {30, 
       FormBox["\"30\[Degree]\"", TraditionalForm]}, {45, 
       FormBox["\"45\[Degree]\"", TraditionalForm]}}, {{0, 
       FormBox["\"\"", TraditionalForm]}, {15, 
       FormBox["\"\"", TraditionalForm]}, {30, 
       FormBox["\"\"", TraditionalForm]}, {45, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.6368471205099077`*^9, 3.636847463561467*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["RG change of the angles:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\) is :    \
\!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\
\))-\!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\)) = \>\"", ",", 
    RowBox[{
     RowBox[{"\[Theta]12", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[Theta]12", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\) is :    \
\!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\
\))-\!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\)) = \>\"", ",", 
    RowBox[{
     RowBox[{"\[Theta]13", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[Theta]13", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";", 
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\) is :    \
\!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\
\))-\!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\)) = \>\"", ",", 
    RowBox[{
     RowBox[{"\[Theta]23", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[Theta]23", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(12\\)]\\) is :    \\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(12\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(12\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "5.486653027558155`"}], 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\) is :    \
\!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\
\))-\!\(\*SubscriptBox[\(\[Theta]\), \(12\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\)) = ", -5.486653027558155, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847120664076*^9, 3.6368474637158937`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(13\\)]\\) is :    \\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(13\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(13\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "0.06985690395314742`"}], 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\) is :    \
\!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\
\))-\!\(\*SubscriptBox[\(\[Theta]\), \(13\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\)) = ", -0.06985690395314742, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847120664076*^9, 3.636847463717544*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(23\\)]\\) is :    \\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(23\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \
\\(23\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "1.96197934516384`"}], 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\) is :    \
\!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\
\))-\!\(\*SubscriptBox[\(\[Theta]\), \(23\)]\)(\!\(\*SubscriptBox[\(M\), \
\(Z\)]\)) = ", -1.96197934516384, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847120664076*^9, 3.6368474637189074`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Phases", "Subsection"],

Cell["Draw the plot for the phases", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Delta]", "[", "t", "]"}], ",", 
       RowBox[{"\[CurlyPhi]1", "[", "t", "]"}], ",", 
       RowBox[{"\[CurlyPhi]2", "[", "t", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\>\""}], "}"}]}], ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "45"}], ",", 
            RowBox[{
             RowBox[{"ToString", "[", 
              RowBox[{"i", "*", "45"}], "]"}], "<>", "\"\<\[Degree]\>\""}]}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "8"}], "}"}]}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "45"}], ",", "\"\<\>\""}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "12"}], "}"}]}], "]"}]}], "}"}]}], ",",
      "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", "Blue", ",", "ForestGreen"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{"\[Delta]", ",", 
            RowBox[{"FontColor", "\[Rule]", "Red"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"\[Delta]", "[", "tmin", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["\[CurlyPhi]", "1"], ",", 
            RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"\[CurlyPhi]1", "[", "tmin", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["\[CurlyPhi]", "2"], ",", 
            RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"\[CurlyPhi]2", "[", "tmin", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", 
            RowBox[{"-", "1"}]}], "}"}]}], "]"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{3.435655636439897*^9, 3.435655679288266*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[1, 0, 0], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03s01HkYx3G3ZkY2VJRKbsn1hzFuRZlPE36DUGJEKFQUFXLpMkbI1u7I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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwl1Xs41FkfAHAhBm2mmNImpMn1h19RxOIb1VhWRTdmh/VExRsVQltRSCnW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     "]]}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV1Xs4VPkfB/BDmhTrsiFF7lFMNeVSEd/vWhkNRbKFJiaXjExuoZJLLrWk
VM6ZGauUSokYbJZ2ZZOw7qTSPZckWWMnxc/S1u9z/jjPeV7P+/M93/fzfc55
jmFAhGewPEEQN+Ci78+iJDaU7zdEqPxZViQQowx2rU+uAJwQUUt7w/KWhItJ
YCfDVtqilsGGwqtg5cxh2l76mu6/jYG9eTo3wfyROsswewL7HLfJpD1Mmrid
7iQw8f3R4GLwy4mPCV1SOUxwXAxugUd2Xrd66KeACSenMQnYOlaD/CcEzEub
oZ0uSvv4XRSY9YBRDjZ8vk/CSQPnOBnT5vrprWwsAje7c2k/4ot1amRg5sku
2vWJmfL5qfMxER95uwKsenmGV5sF9mq8T9uvPuTecwq8eWkP7Tl55wStInBM
4zhtmwz56bPt9LyxaSW4LOfYh1QNBiamv+bSzis62MO/AdZdk/wreDbON8in
HDyx7QxtH2f2zNY74JjwC7S13xkYWLSBHStraAuNHkdIpeDT9jLa5/JtVaOt
F2BiJGrfbbBMYHYtwAGcmhdB232zxoadbDCzMZG2ykupn5UPeFQ7j3aW9hXJ
dAI4rq2X9tj7M47vT4C9ZwZoc2ri+55mg21N/6GtuMvr650CcEy6UhX4JLlg
+7FGsIztRHsk4PNgWCe44qgn7S3rB2O5feCkUh7tQrmuhdv6wcRAOG2Fh3/k
24+CXy1OpB1YULRuzUfwjGMJ7YYIqklvFmxafoO2EUrxUZ2niNfmffyQDE5R
CZd+U1LEhMBKwRg88No3RaahSK83bA4TI1zG1hxcrog/Zuv9EAr+6mpo37BW
ERf+ZJtZfkCMEjIfn07fqYjnbdKy2xwqRq+87xvEchXxOsOc8H6+GNmtlFQF
B8PzPr8qTAXPNv/8yvmwIo47Ory0NUSMjijYMRdeUMSJdfUrdu0Xo0PJV9rP
DMF8rnpTZKAYZVeOeONx8PEVbsrgkrcWI5NTYP65vqIAMRpiVxPeCxfiLLOD
U2/2iZGnWru1EWsh9PfYs40nRqwrny9XJ4K9nFrM94rRRAM7pl97ETbbMWc+
ugu+H8aE7jp3JewdsZF1kCNGjAR3z9Cc77Blzcy0s6UY8XqGJ1TnqeFk6/tz
/GVi1F24e3jdBnXcLd83sUlejEjZY9dLu77HRLTX+MDfIvSO9HuzNXsxnnfy
mWjwkQhlSSaHUKUGJnrKnKrqRMgpa7M4oUsTx+W0Plt9E/J7OXEz85Zg4mr2
qPp5ETIN+/TXnyba+Lqy+v26YyLkHs5zaty7FE9krQweDIJ5y7RItfPLMGH1
afz2dhHSjaxQKSrXwekbXXY62oqQrcLY04QRXZx59NVBHxMRKlNdszhfUQ8X
O5/SMVUTIUl00tEvjvqYcCk9eWVWiAZdG9gP7Qxw2XNHv4p3QqSZ15bV2m6A
CeUW7ZBeIVLT8AkeDDHEmeoDgY11QvRb6/jhS/8aYuLctZCGEiEK4u0ZSsow
wsSdZdv9coXohZHHUJKxMX4Zm1ZakS5EswYcHZMqY7zaw40rjIb9OG0fRj1M
8Kezkb72/kL08KcVag/HTDDBNTmc4SZEgosnIvziV2CiLeXLbjsh6lQRBClp
mWLixq17cyuF6Pnd3Tl7i0zxuQf1Kcu0hWjCVU/7kbUZnK9iWMcCIWpjhFny
e8zwduVP3bxpCnn8d/fnq7yV8D6t0ts3QqF8fWHUly9g5Rrp0icUEiZGU3PZ
q+D9Usld3kQh0V1zVT09c0xo8b81/0YhP7NQfd7vYLfGJaHXKdTTWcpncyyw
u4J1daCIQgsL4tjb31tg4tQDfuBJCvWt3yGr/mCBNeZtM1kBLmyy+EV/3AKv
b0XDIyco5PD3wJhMBvMuvKgD4Ggb12xyFnzzdH1UOoVedOg/efYdE3sbkutT
UilU8m9LQKAVE7+K9Q++lkShw6evKXXYMDGhsEgSDHbST6qy2gTm682Zgd84
WTIYDkx8pCz5amkihb4/l198kw05scy0OoFC8abRMqkvE4+ezyXa4inkulMn
6UgK5Fq28lNxFGLMPa+tSWPiyAVbP9WCG67m/jt9AnLKfDQVbDupGRt7iolt
60Jeq4FXnlcTRJNM7Pi6aYoZS6Hhjd3FlUImvvNWMP9zDIUKBs68l4lhfcc7
7VrwEpZSYMRFJra47eG6Fdz7tLVAcomJNUwdD6iBs5Mz3kgLmNjOjHHm6SHo
0z3fV3Ad1nfpJe8HN8Q1im8VgV/oRq8GJ+mlPRkrZmKuwVn+52jo1/zDYvNS
yOu/9l4ATx0kdoRKmNjg21H/6SgKVWjey75ZQZ/XkikPsKAuseP9r0zsX/eU
vBUJ/ZXn2PtrIL9bN86LgP5Vv5+4/jvkL9/eqA2nEJd75MFwLRMf/sHigBb4
0a0ph8B6mH+8XLlDAP04MmteK9gtq27DAeg3KTl0uZ2JXZ5V1+eEQr+8g5Vv
OiE3kO+U8ilU+WGMubeXieUz/kcUhsB+Ge+MfV8ysWWwV59aMIUSm1+oe0lh
/s4Vm3F/Cm10bJKy1VdjovpRe/FPFLom/6K4ZjH47mujGS8KqTyYCDbTAvN+
SXcGv/1R+w1DZzUuKTLgv/WE890S1tVkAnk9v3y5B4XeuaiX/7hxNW6TnHeg
OBTKcd8bifwh//IHNxNRaJw7NWlTBtbuTyVXUYhz23xdu8MaTAS1z3wloP+Y
7V+pnWDO/OT25yTq3/Zw0oe/Fv5/+P5QOYnujAt2982sxcUvLLr3ZJIoXsHz
+J45yBsnF1VmkMhed0PxwH/gx107GODB68cTY+VYuITxF1F5kkT7n3C6kxks
3K9U+paRTqJIq/5oSo0F3/Ps3aokEp2YXPBHnQnYwKVWOwbmk+K3E6ZgXV2f
iEMkclkkHXI0A3Pn5JqjSbTIqFepZRULR1H+CcFRJDq34+Le3jWQV8tGGsJJ
lFfBknu/kYWncuw7kkNJdGzzNeEqWxYO+xSU95RPIm6LprnADlx16tBasP7A
rOdHexa+t6LHuX8/iQpVmwrnHOF5pbZRKAj6XdiwycGJhXeTbmJxIPQzK+k8
vgVyvm/LPwEkWoXOTs93ARfwXQv2Qb92uSyXrSy8RYOfP8Mj0fiuGP0sDuTp
AbMe4M6hkdudriwsku4KKPYnkSTcx0VtGwvnHNryRA78f/RQgBU=
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 0},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox["\[Delta]", FontColor -> RGBColor[1, 0, 0], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[1, 0, 0], FontSize -> 12]& ], 
      TraditionalForm], {1.9596137110710439`, 18.357881718541215`}, {-0.8, 
     1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[CurlyPhi]", "1"], FontColor -> RGBColor[0, 0, 1], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {1.9596137110710439`, 359.93380534449557`}, {-0.8, 
     1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["\[CurlyPhi]", "2"], FontColor -> 
        RGBColor[0.1, 0.75, 0.2], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {1.9596137110710439`, 
     0.022875423411665707`}, {-0.8, -1}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox["\"\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{0, 
       FormBox["\"0\[Degree]\"", TraditionalForm]}, {45, 
       FormBox["\"45\[Degree]\"", TraditionalForm]}, {90, 
       FormBox["\"90\[Degree]\"", TraditionalForm]}, {135, 
       FormBox["\"135\[Degree]\"", TraditionalForm]}, {180, 
       FormBox["\"180\[Degree]\"", TraditionalForm]}, {225, 
       FormBox["\"225\[Degree]\"", TraditionalForm]}, {270, 
       FormBox["\"270\[Degree]\"", TraditionalForm]}, {315, 
       FormBox["\"315\[Degree]\"", TraditionalForm]}, {360, 
       FormBox["\"360\[Degree]\"", TraditionalForm]}}, {{0, 
       FormBox["\"\"", TraditionalForm]}, {45, 
       FormBox["\"\"", TraditionalForm]}, {90, 
       FormBox["\"\"", TraditionalForm]}, {135, 
       FormBox["\"\"", TraditionalForm]}, {180, 
       FormBox["\"\"", TraditionalForm]}, {225, 
       FormBox["\"\"", TraditionalForm]}, {270, 
       FormBox["\"\"", TraditionalForm]}, {315, 
       FormBox["\"\"", TraditionalForm]}, {360, 
       FormBox["\"\"", TraditionalForm]}, {405, 
       FormBox["\"\"", TraditionalForm]}, {450, 
       FormBox["\"\"", TraditionalForm]}, {495, 
       FormBox["\"\"", TraditionalForm]}, {540, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847121614245*^9, 3.636847464474209*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["RG change of the phases:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \[Delta] is:   \[Delta](\!\(\*SubscriptBox[\(M\), \
\(GUT\)]\))-\[Delta](\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", ",", " ", 
    RowBox[{
     RowBox[{"\[Delta]", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[Delta]", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(1\)]\) is:   \
\!\(\*SubscriptBox[\(\[CurlyPhi]\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \
\(GUT\)]\))-\!\(\*SubscriptBox[\(\[CurlyPhi]\), \
\(1\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", ",", " ", 
    RowBox[{
     RowBox[{"\[CurlyPhi]1", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[CurlyPhi]1", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(2\)]\) is:   \
\!\(\*SubscriptBox[\(\[CurlyPhi]\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \
\(GUT\)]\))-\!\(\*SubscriptBox[\(\[CurlyPhi]\), \
\(2\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", ",", " ", 
    RowBox[{
     RowBox[{"\[CurlyPhi]2", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[CurlyPhi]2", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\<\[Degree]\>\""}], 
   "]"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \[Delta] is:   \
\[Delta](\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\[Delta](\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", "71.60559154569776`", 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "The RG change of \[Delta] is:   \[Delta](\!\(\*SubscriptBox[\(M\), \
\(GUT\)]\))-\[Delta](\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = ", 
   71.60559154569776, "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847122923572*^9, 3.636847464526793*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(\[CurlyPhi]\\), \\(1\
\\)]\\) is:   \\!\\(\\*SubscriptBox[\\(\[CurlyPhi]\\), \
\\(1\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(\[CurlyPhi]\\), \
\\(1\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "0.0199622118753382`"}], 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(1\)]\) is:   \
\!\(\*SubscriptBox[\(\[CurlyPhi]\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \
\(GUT\)]\))-\!\(\*SubscriptBox[\(\[CurlyPhi]\), \
\(1\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = ", -0.0199622118753382, 
   "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847122923572*^9, 3.636847464530242*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(\[CurlyPhi]\\), \\(2\
\\)]\\) is:   \\!\\(\\*SubscriptBox[\\(\[CurlyPhi]\\), \
\\(2\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(\[CurlyPhi]\\), \
\\(2\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "0.0039583125047304435`"}], 
   "\[InvisibleSpace]", "\<\"\[Degree]\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(2\)]\) is:   \
\!\(\*SubscriptBox[\(\[CurlyPhi]\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \
\(GUT\)]\))-\!\(\*SubscriptBox[\(\[CurlyPhi]\), \
\(2\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = ", -0.0039583125047304435`, 
   "\[Degree]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847122923572*^9, 3.636847464533923*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Masses", "Subsection"],

Cell[CellGroupData[{

Cell["mass eigenvalues", "Subsubsection"],

Cell["Draw the plot for the mass eigenvalues", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"tlabel", "=", "4"}], ";"}], "   ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Position", " ", "where", " ", "the", " ", "labels", " ", "m1"}], 
    ",", "m2", ",", 
    RowBox[{"m3", " ", "should", " ", "be", " ", "placed"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"m1", "[", "t", "]"}], ",", 
       RowBox[{"m2", "[", "t", "]"}], ",", 
       RowBox[{"m3", "[", "t", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(m\), \(i\)]\) [eV]\>\""}], "}"}]}], ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "0.01"}], ",", 
            RowBox[{"PaddedForm", "[", 
             RowBox[{
              RowBox[{"i", "*", "0.01"}], ",", 
              RowBox[{"{", 
               RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "50"}], "}"}]}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "0.01"}], ",", "\"\<\>\""}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "50"}], "}"}]}], "]"}]}], "}"}]}], ",",
      "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", "Blue", ",", "ForestGreen"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["m", "1"], ",", 
            RowBox[{"FontColor", "\[Rule]", "Red"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tlabel", ",", 
            RowBox[{"m1", "[", "tlabel", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["m", "2"], ",", 
            RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tlabel", ",", 
            RowBox[{"m2", "[", "tlabel", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", 
            RowBox[{"-", "2"}]}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
        
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubscriptBox["m", "3"], ",", 
            RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tlabel", ",", 
            RowBox[{"m3", "[", "tlabel", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}]}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]}], "Input",
 CellChangeTimes->{3.435655642671777*^9, 3.435655682603495*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[1, 0, 0], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k4lc0bB3DrcSxZQlR2sp0HR7ZSmZuUUCmkrIlE2ZUl2VK9lDoe5zzS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     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0ns0VOsbwHFpzIycIiWK3IvMNkYpIp4nkdBJSYmQlHRRyjlSQjXIrZS9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     "]]}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03k0ldsbB3DVcc5BHESdIvMUr7xlKuF5Eh3RpcTNnAalELkp1xTJpbgS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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 0.012},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["m", "1"], FontColor -> RGBColor[1, 0, 0], FontSize -> 
        12], StyleForm[#, FontColor -> RGBColor[1, 0, 0], FontSize -> 12]& ], 
      TraditionalForm], {4, 0.01661933208461064}, {-0.8, 1.4}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["m", "2"], FontColor -> RGBColor[0, 0, 1], FontSize -> 
        12], StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {4, 0.0193330121557749}, {-0.8, -2}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubscriptBox["m", "3"], FontColor -> RGBColor[0.1, 0.75, 0.2], 
        FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {4, 0.054821183820263436`}, {-0.8, 1.4}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(m\\), \\(i\\)]\\) [eV]\"", TraditionalForm], 
     None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{0., 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.00\"", 0., AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.01, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.01\"", 0.01, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.02, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.02\"", 0.02, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.03, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.03\"", 0.03, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.04, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.04\"", 0.04, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.05, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.05\"", 0.05, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.06, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.06\"", 0.06, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.07, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.07\"", 0.07, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.08, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.08\"", 0.08, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.09, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.09\"", 0.09, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.1, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.10\"", 0.1, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.11, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.11\"", 0.11, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.12, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.12\"", 0.12, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.13, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.13\"", 0.13, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.14, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.14\"", 0.14, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.15, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.15\"", 0.15, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.16, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.16\"", 0.16, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.17, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.17\"", 0.17, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.18, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.18\"", 0.18, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.19, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.19\"", 0.19, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.20\"", 0.2, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.21, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.21\"", 0.21, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.22, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.22\"", 0.22, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.23, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.23\"", 0.23, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.24, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.24\"", 0.24, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.25, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.25\"", 0.25, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.26, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.26\"", 0.26, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.27, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.27\"", 0.27, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.28, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.28\"", 0.28, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.29, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.29\"", 0.29, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.3, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.30\"", 0.3, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.31, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.31\"", 0.31, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.32, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.32\"", 0.32, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.33, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.33\"", 0.33, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.34, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.34\"", 0.34, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.35000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.35\"", 0.35000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.36, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.36\"", 0.36, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.37, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.37\"", 0.37, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.38, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.38\"", 0.38, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.39, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.39\"", 0.39, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.40\"", 0.4, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.41000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.41\"", 0.41000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.42, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.42\"", 0.42, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.43, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.43\"", 0.43, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.44, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.44\"", 0.44, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.45, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.45\"", 0.45, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.46, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.46\"", 0.46, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.47000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.47\"", 0.47000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.48, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.48\"", 0.48, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.49, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.49\"", 0.49, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}, {0.5, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.50\"", 0.5, AutoDelete -> True], 
         PaddedForm[#, {3, 2}]& ], TraditionalForm]}}, {{0., 
       FormBox["\"\"", TraditionalForm]}, {0.01, 
       FormBox["\"\"", TraditionalForm]}, {0.02, 
       FormBox["\"\"", TraditionalForm]}, {0.03, 
       FormBox["\"\"", TraditionalForm]}, {0.04, 
       FormBox["\"\"", TraditionalForm]}, {0.05, 
       FormBox["\"\"", TraditionalForm]}, {0.06, 
       FormBox["\"\"", TraditionalForm]}, {0.07, 
       FormBox["\"\"", TraditionalForm]}, {0.08, 
       FormBox["\"\"", TraditionalForm]}, {0.09, 
       FormBox["\"\"", TraditionalForm]}, {0.1, 
       FormBox["\"\"", TraditionalForm]}, {0.11, 
       FormBox["\"\"", TraditionalForm]}, {0.12, 
       FormBox["\"\"", TraditionalForm]}, {0.13, 
       FormBox["\"\"", TraditionalForm]}, {0.14, 
       FormBox["\"\"", TraditionalForm]}, {0.15, 
       FormBox["\"\"", TraditionalForm]}, {0.16, 
       FormBox["\"\"", TraditionalForm]}, {0.17, 
       FormBox["\"\"", TraditionalForm]}, {0.18, 
       FormBox["\"\"", TraditionalForm]}, {0.19, 
       FormBox["\"\"", TraditionalForm]}, {0.2, 
       FormBox["\"\"", TraditionalForm]}, {0.21, 
       FormBox["\"\"", TraditionalForm]}, {0.22, 
       FormBox["\"\"", TraditionalForm]}, {0.23, 
       FormBox["\"\"", TraditionalForm]}, {0.24, 
       FormBox["\"\"", TraditionalForm]}, {0.25, 
       FormBox["\"\"", TraditionalForm]}, {0.26, 
       FormBox["\"\"", TraditionalForm]}, {0.27, 
       FormBox["\"\"", TraditionalForm]}, {0.28, 
       FormBox["\"\"", TraditionalForm]}, {0.29, 
       FormBox["\"\"", TraditionalForm]}, {0.3, 
       FormBox["\"\"", TraditionalForm]}, {0.31, 
       FormBox["\"\"", TraditionalForm]}, {0.32, 
       FormBox["\"\"", TraditionalForm]}, {0.33, 
       FormBox["\"\"", TraditionalForm]}, {0.34, 
       FormBox["\"\"", TraditionalForm]}, {0.35000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {0.36, 
       FormBox["\"\"", TraditionalForm]}, {0.37, 
       FormBox["\"\"", TraditionalForm]}, {0.38, 
       FormBox["\"\"", TraditionalForm]}, {0.39, 
       FormBox["\"\"", TraditionalForm]}, {0.4, 
       FormBox["\"\"", TraditionalForm]}, {0.41000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {0.42, 
       FormBox["\"\"", TraditionalForm]}, {0.43, 
       FormBox["\"\"", TraditionalForm]}, {0.44, 
       FormBox["\"\"", TraditionalForm]}, {0.45, 
       FormBox["\"\"", TraditionalForm]}, {0.46, 
       FormBox["\"\"", TraditionalForm]}, {0.47000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {0.48, 
       FormBox["\"\"", TraditionalForm]}, {0.49, 
       FormBox["\"\"", TraditionalForm]}, {0.5, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847123706056*^9, 3.636847465241275*^9}]
}, Open  ]],

Cell["RG change of the masses:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(m\), \(1\)]\) is:   \
\!\(\*SubscriptBox[\(m\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\
\*SubscriptBox[\(m\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", 
    ",", 
    RowBox[{
     RowBox[{"m1", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"m1", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\< eV\>\""}], "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(m\), \(2\)]\) is:   \
\!\(\*SubscriptBox[\(m\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\
\*SubscriptBox[\(m\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", 
    ",", 
    RowBox[{
     RowBox[{"m2", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"m2", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\< eV\>\""}], "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[\(m\), \(3\)]\) is:   \
\!\(\*SubscriptBox[\(m\), \(3\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\
\*SubscriptBox[\(m\), \(3\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", 
    ",", 
    RowBox[{
     RowBox[{"m3", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"m3", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", "\"\< eV\>\""}], "]"}],
   ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(m\\), \\(1\\)]\\) \
is:   \\!\\(\\*SubscriptBox[\\(m\\), \
\\(1\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(m\\), \
\\(1\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", "0.005156542987090871`", 
   "\[InvisibleSpace]", "\<\" eV\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(m\), \(1\)]\) is:   \
\!\(\*SubscriptBox[\(m\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\
\*SubscriptBox[\(m\), \(1\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = ", 
   0.005156542987090871, " eV"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847123754135*^9, 3.636847465317541*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(m\\), \\(2\\)]\\) \
is:   \\!\\(\\*SubscriptBox[\\(m\\), \
\\(2\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(m\\), \
\\(2\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", "0.006415547841805449`", 
   "\[InvisibleSpace]", "\<\" eV\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(m\), \(2\)]\) is:   \
\!\(\*SubscriptBox[\(m\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\
\*SubscriptBox[\(m\), \(2\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = ", 
   0.006415547841805449, " eV"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847123754135*^9, 3.636847465321701*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[\\(m\\), \\(3\\)]\\) \
is:   \\!\\(\\*SubscriptBox[\\(m\\), \
\\(3\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[\\(m\\), \
\\(3\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", "0.018620175095885545`", 
   "\[InvisibleSpace]", "\<\" eV\"\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[\(m\), \(3\)]\) is:   \
\!\(\*SubscriptBox[\(m\), \(3\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\
\*SubscriptBox[\(m\), \(3\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = ", 
   0.018620175095885545`, " eV"],
  Editable->False]], "Print",
 CellChangeTimes->{3.636847123754135*^9, 3.636847465346807*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["mass squared differences", "Subsubsection"],

Cell["\<\
Draw the plot for the mass squared differences (on a logarithmic scale)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Log", "[", 
        RowBox[{"10", ",", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[CapitalDelta]m2sol", "[", "t", "]"}], "]"}]}], "]"}], 
       ",", 
       RowBox[{"Log", "[", 
        RowBox[{"10", ",", 
         RowBox[{"Abs", "[", 
          RowBox[{"\[CapitalDelta]m2atm", "[", "t", "]"}], "]"}]}], "]"}]}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\!\(\*SubsuperscriptBox[\(\[CapitalDelta]m\), \(i\), \(2\)]\) \
[\!\(\*SuperscriptBox[\(eV\), \(2\)]\)]\>\""}], "}"}]}], ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"RGELogTicksLabeledNegExp", "[", 
         RowBox[{
          RowBox[{"-", "6"}], ",", 
          RowBox[{"-", "1"}]}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{
          RowBox[{"-", "6"}], ",", 
          RowBox[{"-", "1"}]}], "]"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"ForestGreen", ",", "Blue"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubsuperscriptBox["\[CapitalDelta]m", "sol", "2"], ",", 
            RowBox[{"FontColor", "\[Rule]", "ForestGreen"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"Log", "[", 
             RowBox[{"10", ",", 
              RowBox[{"Abs", "[", 
               RowBox[{"\[CapitalDelta]m2sol", "[", "tmin", "]"}], "]"}]}], 
             "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", 
            RowBox[{"-", "2"}]}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
        
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"StyleForm", "[", 
           RowBox[{
            SubsuperscriptBox["\[CapitalDelta]m", "atm", "2"], ",", 
            RowBox[{"FontColor", "\[Rule]", "Blue"}], ",", 
            RowBox[{"FontSize", "\[Rule]", "12"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"tmin", ",", 
            RowBox[{"Log", "[", 
             RowBox[{"10", ",", 
              RowBox[{"Abs", "[", 
               RowBox[{"\[CapitalDelta]m2atm", "[", "tmin", "]"}], "]"}]}], 
             "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "0.8"}], ",", "1.4"}], "}"}]}], "]"}]}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{3.435655651153079*^9, 3.4361001025721*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV1Hk41WkbB3DEOQfjd35CmWSLsZzzOxx7Q3GPcCxNSiYxjghhKEuR6tCi
Jg2dGvtkSdEriShLqOnJki1LoRnMa8tSwnt4Y0S93mf+eK7n+lz3/cf3ua/7
ejQPh7sFSoiJid3G55/7z8gy8zSvdRtth7oNxHESJfLqPbPC1m2UH5eX1kSR
yEK1VZATv24TutvqhB92RutYQ+HtdRvb5bYDVZEkcldXcq2aWbfZ93A13CeC
RMFTT01Cd4pBWzdnouooiSZStXcnd4qBcs2C36lgEg3NLwi65sRBsNfdw9WH
RFP775i+8pEEvtXfNilOJDKLVkz9T5Ak5CemzvOwL2YkLMhFSoLUutmjL44k
0hzwK3NOkARHC9/4EGxvHzW9piJJuFlPqNjySNQbnKlSI5KEXRe/C/jbjkQo
7opE7gUpyIvgf4gEEjFvrvjWJ0nBPZ96exa2Dwp6NpAmBXNu7ZXjNiRak3AQ
bCqSArZvb89+bPNEieVrHVLgH3hm3dyaRKUpZ95fUKTBQtRpA/EdJLpRdLQn
+F80uG+0Jiy0INFqjFeA5wMaPA8StPhiezrwVpwe02Ck9vFGVWzlSQ0NdjsN
KhVgLN2cROnb+sLn5mgwtJrJTzQj0fVcS2aUGR3uJcx2RJiQSBSmW3DYmg6n
2FsoA2zXHYoW+3l0eGh8Ju+DMYmIoTkfU0869Auqa49gJynfKlsW0OHKCaqN
b0Sin1Ppe8400SE7dsnO1RDP+/DHsdBOOsyc/blJDtveeCza+w0dTGLbfnhp
QCLJV3W5O9/RweE3+1InbP/8IiODBTrU0sOPM7AbwtOa1VbpMC9iubRwSLTN
5rwncwMDfnnzVYwQ+zxxbG5dlgG3rVIuB2KP/tvrvEiRAYpTB/J2YEMpT2lM
lQFmHz3rFLD/56K5s8GQAXNVmyUaKBIJrvQlX9zPgE/Gnj1q2H8dfK4R7c0A
F/4KucwmkZVeWWVgIAOMJrsOdGKvvrj8l8NJBrgL1RbPYMdKWlHS2Qyw7zcM
GmCR6PjZWx1Xxxlg6+bhnK5PImHF1EGYZYDEr7qRIdj33rKnFpcYEFAuk7cT
e5xXLXZQWhpkWVIyU3okciM7zLZxpcHJ4T7LApt76+PN6jhpCHfruDqoQ6L5
Bt6JEWUZyEje1fuNNt5/2vxWI1dZ6L+Wr9mlRiKawNUtJEUO0mq2sBWUSOTb
MzHP3EBCUMFK1ogMiboLPSaMLORh9ekbcyNxEqWK+lzyDmyEnE+N+lMiJppM
9Rl2EiqATX1ddNIwEyWVLY7bVChCUfsD+eVOJrJL2pEp6FICoZAZb/o7rj9L
iVnZsBmWDjdlFZUykU7of1t+11aGO9p3G6vymMj1mK9dE/9rWDY205O5hvtN
EiLIX7eAcXlLCeccE22NKCeKHqiAB9z3fBHFRJaSM38IpraCyuas8NoAJipl
GijkMtSAK8OQZx5korKo+FOfbdVhmbeloseZicZcGnivrDRgMvJ+iLM1Eynd
aE9q69CAHt7HZZoJE5GKnoFjQZqQH6YptNNloqq22ZN5nzQhvqX3UNVWJgrw
/XE8PnEbfG7VoT3byESD2/aOx2tpgZ1++3sfBhOtajiraFdqQZ9S477N6wQa
c25//26vNrBbedtjlwj06odvyFcz2pBdWhoSM0ugsJxL4T6nv4HL+x1vHZ8g
UCcRFiC7SQcqqiv3cv8i0MATjxR+kQ7MFstmj/QRaN5FTbnXTBfUNnDsFLoI
1E4LNQnu0YX2kJoB1RYC7f3y5PJtXz14srzrqORzAuWqp0d+/qwHpqJPJa11
BEqPi0pbE+rD8kdhtXcVgTKesJhqaizweu9cdbScQD66Ieq+tSwoUBqePl9C
oJ7O+8E8ZzZ8aTE/6FZEIOn8GN6eaTbICfitVgUEemO8T1T9ng2d4b3FFHZh
M/s39Vk2XPNPTlHFtv4wOiMSscEktfXU+m0CRZm7CFNX2SBLHr/ZiD34Ur3/
TzkKEmv/5H2Pfe9T62F/UwrsszjMI7cIdDK5QPalOa5bFA54YNupx1eafkuB
tX1wiRP2sJ0JjWZNgUfQYAAHe+P13OK7PAquP440XMon0GmdKNGcFwUDAfVa
P2O77FeJjz1PARlrnlR6k0C0tYH6mgQK3EuDtW9hN9zO+rR8iYKCB+uNadiW
i0rR0b9QYOrO3yTA1vuVDItKpaDy1AjdBXtie3dxRToFFRehzxo7f/TqtCiT
Au9GRpEx9maurH94DgVyzJ5DW7Bf/9GWX5ZHQVF1tz2BLTybODyXT0HuCwtj
iX/ydEt5hd2hwO8HW7WZPJwnpimzpIiC+PBJ1WHseLWE/pliCn7TW9R6jW35
4jsF1n0KtMSvVj/FXjoqti+kDL9v9FFRGna50jPh3XIKzglZuaHYYU/jXk4/
pKDRbyHDFnviqzXekRoKsl5GZIpycf7K2kt3ainoJlh5Ldje3rGNE/UUnOww
Lc7D7i1ZsvZHFHxgwMvd2A3OIjPfNgpips0dinNwvsWy4zc7KEh2vRR+Dtvy
xtGK4U4K7g7p5npgV7yfofivKeBX/iRFw85PnNTyGqJAo0ok7Z9NoLgXg/Lu
cxSoODq+Vb1BoO22zXM8eQ4kSJ4cyMgkUIHEYHGNAgcmpy7qhWITjfOBups4
4KVWLbDBfrtLeZimwoG3lhdN32Xg+dqHdjVrcyD63eDYt9iTjvIPdm3nwJ7Z
bpPhNAKluPIjbA5x4NjZ5hpWCoFmvZcWzUs5IL1B36M3mUDOj1hGHdYG4FF4
9crGBJx/xrLlQqcB5PWdLe04RaCR718tegYbgmD+zQQRQaDHs2Eeb1YMYThs
5JpTEN4/SbdzP64ZwhqVtkULe+dWi+LRL7ge+EvJ5yP4f7hzLi5anAtBcdaj
D7CP9Dt3n6VxIWd4KG0TdoTpSFQayYVpuXz2ZACBLi3S655qc6FkXVrp8mHc
H396j5gOF5qeJoT4YTvKzI3b6nJBlOXZZIUts+21bKs+Fxo2ql4V+RHo+r4c
/msDLpBSKqFe2DfKueLT27lgceyhp6Evgc7sKEjXt+SCdJe7ogy2d6sSK8yK
C6Vf+/VPHCKQ+uiq28JOLmz+u+6nG9iFzObCNVsusH5MHpfCvpRt8a21HRc4
9zJ6Rn1wPt17nefsuTA1K99cj61vc21ZypELK0M+zyOxZTrEkxyduNC8x7ht
N/bsgRPqSc5cqLQ9/Ycuduf41KNOFy50ZVvMSmCXHfN0JL/nQpF1KH2YT6D/
A0ohh1s=
     "]]}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV1Hk41dkfB3B8v997r0yuQhhrEsoXV7ayfiq6YaK06EryS0LIUpaKO4Wa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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., -4.2},
  DisplayFunction->Identity,
  Epilog->{
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubsuperscriptBox["\[CapitalDelta]m", "sol", "2"], FontColor -> 
        RGBColor[0.1, 0.75, 0.2], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0.1, 0.75, 0.2], FontSize -> 12]& ],
       TraditionalForm], {1.9596137110710439`, -4.1123581707372}, {-0.8, -2}], 
    InsetBox[
     FormBox[
      TagBox[
       StyleBox[
        SubsuperscriptBox["\[CapitalDelta]m", "atm", "2"], FontColor -> 
        RGBColor[0, 0, 1], FontSize -> 12], 
       StyleForm[#, FontColor -> RGBColor[0, 0, 1], FontSize -> 12]& ], 
      TraditionalForm], {1.9596137110710439`, -2.679231641337602}, {-0.8, 
     1.4}]},
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubsuperscriptBox[\\(\[CapitalDelta]m\\), \\(i\\), \
\\(2\\)]\\) [\\!\\(\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)]\"", 
      TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{-6, 
       FormBox["\"\\!\\(10\\^\\(-6\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500000], -5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 3]], -5.5228787452803365`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250000], -5.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200000], -5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500000, 3]], -5.221848749616355], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 7]], -5.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125000], -5.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 9]], -5.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-5, 
       FormBox["\"\\!\\(10\\^\\(-5\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50000], -4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 3]], -4.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25000], -4.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20000], -4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50000, 3]], -4.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 7]], -4.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[12500], -4.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 9]], -4.045757490560674], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-4, 
       FormBox["\"\\!\\(10\\^\\(-4\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[5000], -3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 3]], -3.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2500], -3.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2000], -3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[5000, 3]], -3.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 7]], -3.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[1250], -3.096910013008056], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 9]], -3.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-3, 
       FormBox["\"\\!\\(10\\^\\(-3\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500], -2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 3]], -2.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250], -2.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200], -2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500, 3]], -2.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 7]], -2.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125], -2.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 9]], -2.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-2, 
       FormBox["\"\\!\\(10\\^\\(-2\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50], -1.6989700043360185`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 3]], -1.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25], -1.3979400086720373`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20], -1.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50, 3]], -1.2218487496163561`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 7]], -1.1549019599857429`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[25, 2]], -1.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 9]], -1.0457574905606748`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-1, 
       FormBox["\"\\!\\(10\\^\\(-1\\)\\)\"", TraditionalForm]}, {
       NCache[-Log[5]/Log[10], -0.6989700043360186], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 3]]/Log[10], -0.5228787452803375], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 2]]/Log[10], -0.39794000867203755`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[2]/Log[10], -0.30102999566398114`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 3]]/Log[10], -0.22184874961635634`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 7]]/Log[10], -0.15490195998574316`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 4]]/Log[10], -0.0969100130080564], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 9]]/Log[10], -0.04575749056067514], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{-6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500000], -5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 3]], -5.5228787452803365`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250000], -5.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200000], -5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500000, 3]], -5.221848749616355], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 7]], -5.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125000], -5.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000000, 9]], -5.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50000], -4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 3]], -4.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25000], -4.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20000], -4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50000, 3]], -4.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 7]], -4.154901959985742], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[12500], -4.096910013008055], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100000, 9]], -4.045757490560674], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[5000], -3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 3]], -3.522878745280337], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2500], -3.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[2000], -3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[5000, 3]], -3.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 7]], -3.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[1250], -3.096910013008056], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[10000, 9]], -3.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[500], -2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 3]], -2.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[250], -2.397940008672037], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[200], -2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[500, 3]], -2.221848749616356], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 7]], -2.1549019599857426`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[125], -2.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[1000, 9]], -2.045757490560675], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[10]^(-1) Log[50], -1.6989700043360185`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 3]], -1.5228787452803374`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[25], -1.3979400086720373`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[20], -1.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[50, 3]], -1.2218487496163561`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 7]], -1.1549019599857429`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[25, 2]], -1.0969100130080562`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[10]^(-1) Log[
          Rational[100, 9]], -1.0457574905606748`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {-1, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[-Log[5]/Log[10], -0.6989700043360186], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 3]]/Log[10], -0.5228787452803375], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 2]]/Log[10], -0.39794000867203755`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[2]/Log[10], -0.30102999566398114`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 3]]/Log[10], -0.22184874961635634`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 7]]/Log[10], -0.15490195998574316`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[5, 4]]/Log[10], -0.0969100130080564], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[-Log[
          Rational[10, 9]]/Log[10], -0.04575749056067514], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847124652256*^9, 3.6368474661977262`*^9}]
}, Open  ]],

Cell["\<\
Draw the plot for the solar mass squared difference (on a linear scale)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]m2sol", "[", "t", "]"}], "*", 
      RowBox[{"10", "^", "5"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\!\(\*SubsuperscriptBox[\(\[CapitalDelta]m\), \(sol\), \(2\)]\) \
[\!\(\*SuperscriptBox[\(10\), \
\(-5\)]\)\[ThinSpace]\!\(\*SuperscriptBox[\(eV\), \(2\)]\)]\>\""}], "}"}]}], 
     ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"i", ",", "i"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"-", "100"}], ",", "100"}], "}"}]}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"i", ",", "\"\<\>\""}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"-", "100"}], ",", "100"}], "}"}]}], "]"}]}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", "ForestGreen", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{3.435655655075964*^9, 3.435655686756006*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0.1, 0.75, 0.2], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwVknk0VXsbxwnnHClDxqbD2Y7x7H32dg2J8K3kuCTRSFKXhlNU0pUGJE26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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 7.5},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubsuperscriptBox[\\(\[CapitalDelta]m\\), \\(sol\\), \
\\(2\\)]\\) [\\!\\(\\*SuperscriptBox[\\(10\\), \\(-5\\)]\\)\[ThinSpace]\\!\\(\
\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)]\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{-100, 
       FormBox[
        RowBox[{"-", "100"}], TraditionalForm]}, {-99, 
       FormBox[
        RowBox[{"-", "99"}], TraditionalForm]}, {-98, 
       FormBox[
        RowBox[{"-", "98"}], TraditionalForm]}, {-97, 
       FormBox[
        RowBox[{"-", "97"}], TraditionalForm]}, {-96, 
       FormBox[
        RowBox[{"-", "96"}], TraditionalForm]}, {-95, 
       FormBox[
        RowBox[{"-", "95"}], TraditionalForm]}, {-94, 
       FormBox[
        RowBox[{"-", "94"}], TraditionalForm]}, {-93, 
       FormBox[
        RowBox[{"-", "93"}], TraditionalForm]}, {-92, 
       FormBox[
        RowBox[{"-", "92"}], TraditionalForm]}, {-91, 
       FormBox[
        RowBox[{"-", "91"}], TraditionalForm]}, {-90, 
       FormBox[
        RowBox[{"-", "90"}], TraditionalForm]}, {-89, 
       FormBox[
        RowBox[{"-", "89"}], TraditionalForm]}, {-88, 
       FormBox[
        RowBox[{"-", "88"}], TraditionalForm]}, {-87, 
       FormBox[
        RowBox[{"-", "87"}], TraditionalForm]}, {-86, 
       FormBox[
        RowBox[{"-", "86"}], TraditionalForm]}, {-85, 
       FormBox[
        RowBox[{"-", "85"}], TraditionalForm]}, {-84, 
       FormBox[
        RowBox[{"-", "84"}], TraditionalForm]}, {-83, 
       FormBox[
        RowBox[{"-", "83"}], TraditionalForm]}, {-82, 
       FormBox[
        RowBox[{"-", "82"}], TraditionalForm]}, {-81, 
       FormBox[
        RowBox[{"-", "81"}], TraditionalForm]}, {-80, 
       FormBox[
        RowBox[{"-", "80"}], TraditionalForm]}, {-79, 
       FormBox[
        RowBox[{"-", "79"}], TraditionalForm]}, {-78, 
       FormBox[
        RowBox[{"-", "78"}], TraditionalForm]}, {-77, 
       FormBox[
        RowBox[{"-", "77"}], TraditionalForm]}, {-76, 
       FormBox[
        RowBox[{"-", "76"}], TraditionalForm]}, {-75, 
       FormBox[
        RowBox[{"-", "75"}], TraditionalForm]}, {-74, 
       FormBox[
        RowBox[{"-", "74"}], TraditionalForm]}, {-73, 
       FormBox[
        RowBox[{"-", "73"}], TraditionalForm]}, {-72, 
       FormBox[
        RowBox[{"-", "72"}], TraditionalForm]}, {-71, 
       FormBox[
        RowBox[{"-", "71"}], TraditionalForm]}, {-70, 
       FormBox[
        RowBox[{"-", "70"}], TraditionalForm]}, {-69, 
       FormBox[
        RowBox[{"-", "69"}], TraditionalForm]}, {-68, 
       FormBox[
        RowBox[{"-", "68"}], TraditionalForm]}, {-67, 
       FormBox[
        RowBox[{"-", "67"}], TraditionalForm]}, {-66, 
       FormBox[
        RowBox[{"-", "66"}], TraditionalForm]}, {-65, 
       FormBox[
        RowBox[{"-", "65"}], TraditionalForm]}, {-64, 
       FormBox[
        RowBox[{"-", "64"}], TraditionalForm]}, {-63, 
       FormBox[
        RowBox[{"-", "63"}], TraditionalForm]}, {-62, 
       FormBox[
        RowBox[{"-", "62"}], TraditionalForm]}, {-61, 
       FormBox[
        RowBox[{"-", "61"}], TraditionalForm]}, {-60, 
       FormBox[
        RowBox[{"-", "60"}], TraditionalForm]}, {-59, 
       FormBox[
        RowBox[{"-", "59"}], TraditionalForm]}, {-58, 
       FormBox[
        RowBox[{"-", "58"}], TraditionalForm]}, {-57, 
       FormBox[
        RowBox[{"-", "57"}], TraditionalForm]}, {-56, 
       FormBox[
        RowBox[{"-", "56"}], TraditionalForm]}, {-55, 
       FormBox[
        RowBox[{"-", "55"}], TraditionalForm]}, {-54, 
       FormBox[
        RowBox[{"-", "54"}], TraditionalForm]}, {-53, 
       FormBox[
        RowBox[{"-", "53"}], TraditionalForm]}, {-52, 
       FormBox[
        RowBox[{"-", "52"}], TraditionalForm]}, {-51, 
       FormBox[
        RowBox[{"-", "51"}], TraditionalForm]}, {-50, 
       FormBox[
        RowBox[{"-", "50"}], TraditionalForm]}, {-49, 
       FormBox[
        RowBox[{"-", "49"}], TraditionalForm]}, {-48, 
       FormBox[
        RowBox[{"-", "48"}], TraditionalForm]}, {-47, 
       FormBox[
        RowBox[{"-", "47"}], TraditionalForm]}, {-46, 
       FormBox[
        RowBox[{"-", "46"}], TraditionalForm]}, {-45, 
       FormBox[
        RowBox[{"-", "45"}], TraditionalForm]}, {-44, 
       FormBox[
        RowBox[{"-", "44"}], TraditionalForm]}, {-43, 
       FormBox[
        RowBox[{"-", "43"}], TraditionalForm]}, {-42, 
       FormBox[
        RowBox[{"-", "42"}], TraditionalForm]}, {-41, 
       FormBox[
        RowBox[{"-", "41"}], TraditionalForm]}, {-40, 
       FormBox[
        RowBox[{"-", "40"}], TraditionalForm]}, {-39, 
       FormBox[
        RowBox[{"-", "39"}], TraditionalForm]}, {-38, 
       FormBox[
        RowBox[{"-", "38"}], TraditionalForm]}, {-37, 
       FormBox[
        RowBox[{"-", "37"}], TraditionalForm]}, {-36, 
       FormBox[
        RowBox[{"-", "36"}], TraditionalForm]}, {-35, 
       FormBox[
        RowBox[{"-", "35"}], TraditionalForm]}, {-34, 
       FormBox[
        RowBox[{"-", "34"}], TraditionalForm]}, {-33, 
       FormBox[
        RowBox[{"-", "33"}], TraditionalForm]}, {-32, 
       FormBox[
        RowBox[{"-", "32"}], TraditionalForm]}, {-31, 
       FormBox[
        RowBox[{"-", "31"}], TraditionalForm]}, {-30, 
       FormBox[
        RowBox[{"-", "30"}], TraditionalForm]}, {-29, 
       FormBox[
        RowBox[{"-", "29"}], TraditionalForm]}, {-28, 
       FormBox[
        RowBox[{"-", "28"}], TraditionalForm]}, {-27, 
       FormBox[
        RowBox[{"-", "27"}], TraditionalForm]}, {-26, 
       FormBox[
        RowBox[{"-", "26"}], TraditionalForm]}, {-25, 
       FormBox[
        RowBox[{"-", "25"}], TraditionalForm]}, {-24, 
       FormBox[
        RowBox[{"-", "24"}], TraditionalForm]}, {-23, 
       FormBox[
        RowBox[{"-", "23"}], TraditionalForm]}, {-22, 
       FormBox[
        RowBox[{"-", "22"}], TraditionalForm]}, {-21, 
       FormBox[
        RowBox[{"-", "21"}], TraditionalForm]}, {-20, 
       FormBox[
        RowBox[{"-", "20"}], TraditionalForm]}, {-19, 
       FormBox[
        RowBox[{"-", "19"}], TraditionalForm]}, {-18, 
       FormBox[
        RowBox[{"-", "18"}], TraditionalForm]}, {-17, 
       FormBox[
        RowBox[{"-", "17"}], TraditionalForm]}, {-16, 
       FormBox[
        RowBox[{"-", "16"}], TraditionalForm]}, {-15, 
       FormBox[
        RowBox[{"-", "15"}], TraditionalForm]}, {-14, 
       FormBox[
        RowBox[{"-", "14"}], TraditionalForm]}, {-13, 
       FormBox[
        RowBox[{"-", "13"}], TraditionalForm]}, {-12, 
       FormBox[
        RowBox[{"-", "12"}], TraditionalForm]}, {-11, 
       FormBox[
        RowBox[{"-", "11"}], TraditionalForm]}, {-10, 
       FormBox[
        RowBox[{"-", "10"}], TraditionalForm]}, {-9, 
       FormBox[
        RowBox[{"-", "9"}], TraditionalForm]}, {-8, 
       FormBox[
        RowBox[{"-", "8"}], TraditionalForm]}, {-7, 
       FormBox[
        RowBox[{"-", "7"}], TraditionalForm]}, {-6, 
       FormBox[
        RowBox[{"-", "6"}], TraditionalForm]}, {-5, 
       FormBox[
        RowBox[{"-", "5"}], TraditionalForm]}, {-4, 
       FormBox[
        RowBox[{"-", "4"}], TraditionalForm]}, {-3, 
       FormBox[
        RowBox[{"-", "3"}], TraditionalForm]}, {-2, 
       FormBox[
        RowBox[{"-", "2"}], TraditionalForm]}, {-1, 
       FormBox[
        RowBox[{"-", "1"}], TraditionalForm]}, {0, 
       FormBox["0", TraditionalForm]}, {1, 
       FormBox["1", TraditionalForm]}, {2, 
       FormBox["2", TraditionalForm]}, {3, 
       FormBox["3", TraditionalForm]}, {4, 
       FormBox["4", TraditionalForm]}, {5, 
       FormBox["5", TraditionalForm]}, {6, 
       FormBox["6", TraditionalForm]}, {7, 
       FormBox["7", TraditionalForm]}, {8, 
       FormBox["8", TraditionalForm]}, {9, 
       FormBox["9", TraditionalForm]}, {10, 
       FormBox["10", TraditionalForm]}, {11, 
       FormBox["11", TraditionalForm]}, {12, 
       FormBox["12", TraditionalForm]}, {13, 
       FormBox["13", TraditionalForm]}, {14, 
       FormBox["14", TraditionalForm]}, {15, 
       FormBox["15", TraditionalForm]}, {16, 
       FormBox["16", TraditionalForm]}, {17, 
       FormBox["17", TraditionalForm]}, {18, 
       FormBox["18", TraditionalForm]}, {19, 
       FormBox["19", TraditionalForm]}, {20, 
       FormBox["20", TraditionalForm]}, {21, 
       FormBox["21", TraditionalForm]}, {22, 
       FormBox["22", TraditionalForm]}, {23, 
       FormBox["23", TraditionalForm]}, {24, 
       FormBox["24", TraditionalForm]}, {25, 
       FormBox["25", TraditionalForm]}, {26, 
       FormBox["26", TraditionalForm]}, {27, 
       FormBox["27", TraditionalForm]}, {28, 
       FormBox["28", TraditionalForm]}, {29, 
       FormBox["29", TraditionalForm]}, {30, 
       FormBox["30", TraditionalForm]}, {31, 
       FormBox["31", TraditionalForm]}, {32, 
       FormBox["32", TraditionalForm]}, {33, 
       FormBox["33", TraditionalForm]}, {34, 
       FormBox["34", TraditionalForm]}, {35, 
       FormBox["35", TraditionalForm]}, {36, 
       FormBox["36", TraditionalForm]}, {37, 
       FormBox["37", TraditionalForm]}, {38, 
       FormBox["38", TraditionalForm]}, {39, 
       FormBox["39", TraditionalForm]}, {40, 
       FormBox["40", TraditionalForm]}, {41, 
       FormBox["41", TraditionalForm]}, {42, 
       FormBox["42", TraditionalForm]}, {43, 
       FormBox["43", TraditionalForm]}, {44, 
       FormBox["44", TraditionalForm]}, {45, 
       FormBox["45", TraditionalForm]}, {46, 
       FormBox["46", TraditionalForm]}, {47, 
       FormBox["47", TraditionalForm]}, {48, 
       FormBox["48", TraditionalForm]}, {49, 
       FormBox["49", TraditionalForm]}, {50, 
       FormBox["50", TraditionalForm]}, {51, 
       FormBox["51", TraditionalForm]}, {52, 
       FormBox["52", TraditionalForm]}, {53, 
       FormBox["53", TraditionalForm]}, {54, 
       FormBox["54", TraditionalForm]}, {55, 
       FormBox["55", TraditionalForm]}, {56, 
       FormBox["56", TraditionalForm]}, {57, 
       FormBox["57", TraditionalForm]}, {58, 
       FormBox["58", TraditionalForm]}, {59, 
       FormBox["59", TraditionalForm]}, {60, 
       FormBox["60", TraditionalForm]}, {61, 
       FormBox["61", TraditionalForm]}, {62, 
       FormBox["62", TraditionalForm]}, {63, 
       FormBox["63", TraditionalForm]}, {64, 
       FormBox["64", TraditionalForm]}, {65, 
       FormBox["65", TraditionalForm]}, {66, 
       FormBox["66", TraditionalForm]}, {67, 
       FormBox["67", TraditionalForm]}, {68, 
       FormBox["68", TraditionalForm]}, {69, 
       FormBox["69", TraditionalForm]}, {70, 
       FormBox["70", TraditionalForm]}, {71, 
       FormBox["71", TraditionalForm]}, {72, 
       FormBox["72", TraditionalForm]}, {73, 
       FormBox["73", TraditionalForm]}, {74, 
       FormBox["74", TraditionalForm]}, {75, 
       FormBox["75", TraditionalForm]}, {76, 
       FormBox["76", TraditionalForm]}, {77, 
       FormBox["77", TraditionalForm]}, {78, 
       FormBox["78", TraditionalForm]}, {79, 
       FormBox["79", TraditionalForm]}, {80, 
       FormBox["80", TraditionalForm]}, {81, 
       FormBox["81", TraditionalForm]}, {82, 
       FormBox["82", TraditionalForm]}, {83, 
       FormBox["83", TraditionalForm]}, {84, 
       FormBox["84", TraditionalForm]}, {85, 
       FormBox["85", TraditionalForm]}, {86, 
       FormBox["86", TraditionalForm]}, {87, 
       FormBox["87", TraditionalForm]}, {88, 
       FormBox["88", TraditionalForm]}, {89, 
       FormBox["89", TraditionalForm]}, {90, 
       FormBox["90", TraditionalForm]}, {91, 
       FormBox["91", TraditionalForm]}, {92, 
       FormBox["92", TraditionalForm]}, {93, 
       FormBox["93", TraditionalForm]}, {94, 
       FormBox["94", TraditionalForm]}, {95, 
       FormBox["95", TraditionalForm]}, {96, 
       FormBox["96", TraditionalForm]}, {97, 
       FormBox["97", TraditionalForm]}, {98, 
       FormBox["98", TraditionalForm]}, {99, 
       FormBox["99", TraditionalForm]}, {100, 
       FormBox["100", TraditionalForm]}}, {{-100, 
       FormBox["\"\"", TraditionalForm]}, {-99, 
       FormBox["\"\"", TraditionalForm]}, {-98, 
       FormBox["\"\"", TraditionalForm]}, {-97, 
       FormBox["\"\"", TraditionalForm]}, {-96, 
       FormBox["\"\"", TraditionalForm]}, {-95, 
       FormBox["\"\"", TraditionalForm]}, {-94, 
       FormBox["\"\"", TraditionalForm]}, {-93, 
       FormBox["\"\"", TraditionalForm]}, {-92, 
       FormBox["\"\"", TraditionalForm]}, {-91, 
       FormBox["\"\"", TraditionalForm]}, {-90, 
       FormBox["\"\"", TraditionalForm]}, {-89, 
       FormBox["\"\"", TraditionalForm]}, {-88, 
       FormBox["\"\"", TraditionalForm]}, {-87, 
       FormBox["\"\"", TraditionalForm]}, {-86, 
       FormBox["\"\"", TraditionalForm]}, {-85, 
       FormBox["\"\"", TraditionalForm]}, {-84, 
       FormBox["\"\"", TraditionalForm]}, {-83, 
       FormBox["\"\"", TraditionalForm]}, {-82, 
       FormBox["\"\"", TraditionalForm]}, {-81, 
       FormBox["\"\"", TraditionalForm]}, {-80, 
       FormBox["\"\"", TraditionalForm]}, {-79, 
       FormBox["\"\"", TraditionalForm]}, {-78, 
       FormBox["\"\"", TraditionalForm]}, {-77, 
       FormBox["\"\"", TraditionalForm]}, {-76, 
       FormBox["\"\"", TraditionalForm]}, {-75, 
       FormBox["\"\"", TraditionalForm]}, {-74, 
       FormBox["\"\"", TraditionalForm]}, {-73, 
       FormBox["\"\"", TraditionalForm]}, {-72, 
       FormBox["\"\"", TraditionalForm]}, {-71, 
       FormBox["\"\"", TraditionalForm]}, {-70, 
       FormBox["\"\"", TraditionalForm]}, {-69, 
       FormBox["\"\"", TraditionalForm]}, {-68, 
       FormBox["\"\"", TraditionalForm]}, {-67, 
       FormBox["\"\"", TraditionalForm]}, {-66, 
       FormBox["\"\"", TraditionalForm]}, {-65, 
       FormBox["\"\"", TraditionalForm]}, {-64, 
       FormBox["\"\"", TraditionalForm]}, {-63, 
       FormBox["\"\"", TraditionalForm]}, {-62, 
       FormBox["\"\"", TraditionalForm]}, {-61, 
       FormBox["\"\"", TraditionalForm]}, {-60, 
       FormBox["\"\"", TraditionalForm]}, {-59, 
       FormBox["\"\"", TraditionalForm]}, {-58, 
       FormBox["\"\"", TraditionalForm]}, {-57, 
       FormBox["\"\"", TraditionalForm]}, {-56, 
       FormBox["\"\"", TraditionalForm]}, {-55, 
       FormBox["\"\"", TraditionalForm]}, {-54, 
       FormBox["\"\"", TraditionalForm]}, {-53, 
       FormBox["\"\"", TraditionalForm]}, {-52, 
       FormBox["\"\"", TraditionalForm]}, {-51, 
       FormBox["\"\"", TraditionalForm]}, {-50, 
       FormBox["\"\"", TraditionalForm]}, {-49, 
       FormBox["\"\"", TraditionalForm]}, {-48, 
       FormBox["\"\"", TraditionalForm]}, {-47, 
       FormBox["\"\"", TraditionalForm]}, {-46, 
       FormBox["\"\"", TraditionalForm]}, {-45, 
       FormBox["\"\"", TraditionalForm]}, {-44, 
       FormBox["\"\"", TraditionalForm]}, {-43, 
       FormBox["\"\"", TraditionalForm]}, {-42, 
       FormBox["\"\"", TraditionalForm]}, {-41, 
       FormBox["\"\"", TraditionalForm]}, {-40, 
       FormBox["\"\"", TraditionalForm]}, {-39, 
       FormBox["\"\"", TraditionalForm]}, {-38, 
       FormBox["\"\"", TraditionalForm]}, {-37, 
       FormBox["\"\"", TraditionalForm]}, {-36, 
       FormBox["\"\"", TraditionalForm]}, {-35, 
       FormBox["\"\"", TraditionalForm]}, {-34, 
       FormBox["\"\"", TraditionalForm]}, {-33, 
       FormBox["\"\"", TraditionalForm]}, {-32, 
       FormBox["\"\"", TraditionalForm]}, {-31, 
       FormBox["\"\"", TraditionalForm]}, {-30, 
       FormBox["\"\"", TraditionalForm]}, {-29, 
       FormBox["\"\"", TraditionalForm]}, {-28, 
       FormBox["\"\"", TraditionalForm]}, {-27, 
       FormBox["\"\"", TraditionalForm]}, {-26, 
       FormBox["\"\"", TraditionalForm]}, {-25, 
       FormBox["\"\"", TraditionalForm]}, {-24, 
       FormBox["\"\"", TraditionalForm]}, {-23, 
       FormBox["\"\"", TraditionalForm]}, {-22, 
       FormBox["\"\"", TraditionalForm]}, {-21, 
       FormBox["\"\"", TraditionalForm]}, {-20, 
       FormBox["\"\"", TraditionalForm]}, {-19, 
       FormBox["\"\"", TraditionalForm]}, {-18, 
       FormBox["\"\"", TraditionalForm]}, {-17, 
       FormBox["\"\"", TraditionalForm]}, {-16, 
       FormBox["\"\"", TraditionalForm]}, {-15, 
       FormBox["\"\"", TraditionalForm]}, {-14, 
       FormBox["\"\"", TraditionalForm]}, {-13, 
       FormBox["\"\"", TraditionalForm]}, {-12, 
       FormBox["\"\"", TraditionalForm]}, {-11, 
       FormBox["\"\"", TraditionalForm]}, {-10, 
       FormBox["\"\"", TraditionalForm]}, {-9, 
       FormBox["\"\"", TraditionalForm]}, {-8, 
       FormBox["\"\"", TraditionalForm]}, {-7, 
       FormBox["\"\"", TraditionalForm]}, {-6, 
       FormBox["\"\"", TraditionalForm]}, {-5, 
       FormBox["\"\"", TraditionalForm]}, {-4, 
       FormBox["\"\"", TraditionalForm]}, {-3, 
       FormBox["\"\"", TraditionalForm]}, {-2, 
       FormBox["\"\"", TraditionalForm]}, {-1, 
       FormBox["\"\"", TraditionalForm]}, {0, 
       FormBox["\"\"", TraditionalForm]}, {1, 
       FormBox["\"\"", TraditionalForm]}, {2, 
       FormBox["\"\"", TraditionalForm]}, {3, 
       FormBox["\"\"", TraditionalForm]}, {4, 
       FormBox["\"\"", TraditionalForm]}, {5, 
       FormBox["\"\"", TraditionalForm]}, {6, 
       FormBox["\"\"", TraditionalForm]}, {7, 
       FormBox["\"\"", TraditionalForm]}, {8, 
       FormBox["\"\"", TraditionalForm]}, {9, 
       FormBox["\"\"", TraditionalForm]}, {10, 
       FormBox["\"\"", TraditionalForm]}, {11, 
       FormBox["\"\"", TraditionalForm]}, {12, 
       FormBox["\"\"", TraditionalForm]}, {13, 
       FormBox["\"\"", TraditionalForm]}, {14, 
       FormBox["\"\"", TraditionalForm]}, {15, 
       FormBox["\"\"", TraditionalForm]}, {16, 
       FormBox["\"\"", TraditionalForm]}, {17, 
       FormBox["\"\"", TraditionalForm]}, {18, 
       FormBox["\"\"", TraditionalForm]}, {19, 
       FormBox["\"\"", TraditionalForm]}, {20, 
       FormBox["\"\"", TraditionalForm]}, {21, 
       FormBox["\"\"", TraditionalForm]}, {22, 
       FormBox["\"\"", TraditionalForm]}, {23, 
       FormBox["\"\"", TraditionalForm]}, {24, 
       FormBox["\"\"", TraditionalForm]}, {25, 
       FormBox["\"\"", TraditionalForm]}, {26, 
       FormBox["\"\"", TraditionalForm]}, {27, 
       FormBox["\"\"", TraditionalForm]}, {28, 
       FormBox["\"\"", TraditionalForm]}, {29, 
       FormBox["\"\"", TraditionalForm]}, {30, 
       FormBox["\"\"", TraditionalForm]}, {31, 
       FormBox["\"\"", TraditionalForm]}, {32, 
       FormBox["\"\"", TraditionalForm]}, {33, 
       FormBox["\"\"", TraditionalForm]}, {34, 
       FormBox["\"\"", TraditionalForm]}, {35, 
       FormBox["\"\"", TraditionalForm]}, {36, 
       FormBox["\"\"", TraditionalForm]}, {37, 
       FormBox["\"\"", TraditionalForm]}, {38, 
       FormBox["\"\"", TraditionalForm]}, {39, 
       FormBox["\"\"", TraditionalForm]}, {40, 
       FormBox["\"\"", TraditionalForm]}, {41, 
       FormBox["\"\"", TraditionalForm]}, {42, 
       FormBox["\"\"", TraditionalForm]}, {43, 
       FormBox["\"\"", TraditionalForm]}, {44, 
       FormBox["\"\"", TraditionalForm]}, {45, 
       FormBox["\"\"", TraditionalForm]}, {46, 
       FormBox["\"\"", TraditionalForm]}, {47, 
       FormBox["\"\"", TraditionalForm]}, {48, 
       FormBox["\"\"", TraditionalForm]}, {49, 
       FormBox["\"\"", TraditionalForm]}, {50, 
       FormBox["\"\"", TraditionalForm]}, {51, 
       FormBox["\"\"", TraditionalForm]}, {52, 
       FormBox["\"\"", TraditionalForm]}, {53, 
       FormBox["\"\"", TraditionalForm]}, {54, 
       FormBox["\"\"", TraditionalForm]}, {55, 
       FormBox["\"\"", TraditionalForm]}, {56, 
       FormBox["\"\"", TraditionalForm]}, {57, 
       FormBox["\"\"", TraditionalForm]}, {58, 
       FormBox["\"\"", TraditionalForm]}, {59, 
       FormBox["\"\"", TraditionalForm]}, {60, 
       FormBox["\"\"", TraditionalForm]}, {61, 
       FormBox["\"\"", TraditionalForm]}, {62, 
       FormBox["\"\"", TraditionalForm]}, {63, 
       FormBox["\"\"", TraditionalForm]}, {64, 
       FormBox["\"\"", TraditionalForm]}, {65, 
       FormBox["\"\"", TraditionalForm]}, {66, 
       FormBox["\"\"", TraditionalForm]}, {67, 
       FormBox["\"\"", TraditionalForm]}, {68, 
       FormBox["\"\"", TraditionalForm]}, {69, 
       FormBox["\"\"", TraditionalForm]}, {70, 
       FormBox["\"\"", TraditionalForm]}, {71, 
       FormBox["\"\"", TraditionalForm]}, {72, 
       FormBox["\"\"", TraditionalForm]}, {73, 
       FormBox["\"\"", TraditionalForm]}, {74, 
       FormBox["\"\"", TraditionalForm]}, {75, 
       FormBox["\"\"", TraditionalForm]}, {76, 
       FormBox["\"\"", TraditionalForm]}, {77, 
       FormBox["\"\"", TraditionalForm]}, {78, 
       FormBox["\"\"", TraditionalForm]}, {79, 
       FormBox["\"\"", TraditionalForm]}, {80, 
       FormBox["\"\"", TraditionalForm]}, {81, 
       FormBox["\"\"", TraditionalForm]}, {82, 
       FormBox["\"\"", TraditionalForm]}, {83, 
       FormBox["\"\"", TraditionalForm]}, {84, 
       FormBox["\"\"", TraditionalForm]}, {85, 
       FormBox["\"\"", TraditionalForm]}, {86, 
       FormBox["\"\"", TraditionalForm]}, {87, 
       FormBox["\"\"", TraditionalForm]}, {88, 
       FormBox["\"\"", TraditionalForm]}, {89, 
       FormBox["\"\"", TraditionalForm]}, {90, 
       FormBox["\"\"", TraditionalForm]}, {91, 
       FormBox["\"\"", TraditionalForm]}, {92, 
       FormBox["\"\"", TraditionalForm]}, {93, 
       FormBox["\"\"", TraditionalForm]}, {94, 
       FormBox["\"\"", TraditionalForm]}, {95, 
       FormBox["\"\"", TraditionalForm]}, {96, 
       FormBox["\"\"", TraditionalForm]}, {97, 
       FormBox["\"\"", TraditionalForm]}, {98, 
       FormBox["\"\"", TraditionalForm]}, {99, 
       FormBox["\"\"", TraditionalForm]}, {100, 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847125276335*^9, 3.636847466767321*^9}]
}, Open  ]],

Cell["\<\
Draw the plot for the atmospheric mass squared difference (on a linear scale)\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]m2atm", "[", "t", "]"}], "*", 
      RowBox[{"10", "^", "3"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "tmin", ",", "tmax"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\!\(\*SubsuperscriptBox[\(\[CapitalDelta]m\), \(atm\), \(2\)]\) \
[\!\(\*SuperscriptBox[\(10\), \
\(-3\)]\)\[ThinSpace]\!\(\*SuperscriptBox[\(eV\), \(2\)]\)]\>\""}], "}"}]}], 
     ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "0.2"}], ",", 
            RowBox[{"PaddedForm", "[", 
             RowBox[{
              RowBox[{"i", "*", "0.2"}], ",", 
              RowBox[{"{", 
               RowBox[{"2", ",", "1"}], "}"}]}], "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"-", "50"}], ",", "50"}], "}"}]}], "]"}], ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"i", "*", "0.2"}], ",", "\"\<\>\""}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "50"}], "}"}]}], "]"}]}], "}"}]}], ",",
      "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", "Blue", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{3.435655660381*^9, 3.435655690904253*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0nk0lVsDx/HjPMc5R3gehVLKnCEPnTKW8LtSR9xSpBByK8U1FCUlQ0gT
qUxJlESGIppLbjJUhiNK3agU1XEJHSq3uOV93j/22uuz9l5rf/deW3PTdhc/
NovFymfG/+eXoeXm6Z6TthqtHgW7/mLhsLDKIyto0tZs2k7NGw9YsJjzOCon
ZtLWMt6T/lbPQubjntqC/EnbF1rTB3c2s7BWXdn5xsCkbXZs1/Sglyz4i6tN
Aq1ZuGvCsRCOsvAhTef3ZBELX4KSU0vmSuHV8EhU65AUJFNzrs9NkoLYtdC0
3YeDdUczdiT/wYZZuFLa520cWChennpuCxsHMhNG5EM5KBv69HvlNjY0O/8o
d0zgYKG7U2pHMBtePmr69UUcDP8l3DA9ko1n/qdUb0k4eNhjO34wlY2a6CPs
3HhpdHjyeXPq2KDOffetSpIGZ5VkpvZDNnxqtt3vTJdG5NUXjfqNbEywl0dN
L5LGJpfPLJMnbJgfZo8db5bGo8AjrXav2ChL3dcfr8TFntWJP5y/sJFdFNzm
f5GL9jILLU1tAuO7Pbd4XOHCvn+WnJwuAY/lwu8rbnOReXf56TF9AiofNTQM
m7jId6rNbTImkKHVsX1oiAvxRrc1IYsJnMhdTIWZ8bBPJXJnwRoCkiC9C5ts
eFj2sjj7yFoCzkuULFyFPGQdsd8asp4A+WrIx9SDh8lu3idzbwJJKufLx6J4
KO2+eLnOn8DBNN6qffU8hIffVm+JJSDe9LUnUMRDS1VfxqV4AssW9oR7veDB
NfR05dFEApz2u7nW//BwsOpxpzCJwOa8ogXGIzz4zfIXz00hULs9vUFtnIeX
x0RZxEkCWrZxHhTBR8PYgGNkGoE4MmRoUpaPwoylqYYZBN698YyTKPGRrLq8
7nUmAZQJlXvm8DESKnlzLIvALydN69r5fKRbmbZ9PkMg6khH8gFXPuYb2RRf
zSfw2v2BRrgXHwGzluluLiBgpV9+3c+Pj3WO748pXmTe++Gh18sj+PCzfTNz
VwmBPRwrWuYMH3ZuKTqmFQR2xp5vPtbLh7GnOlFRTSClUuyOQT6U/VzyE+4T
KH1vKB79xse9Zpbe+gcEeoU3We4yMpiR3PDuZz0BF4VmMy2BDK4sK1Z2bCEg
OP/13M1oGWwdrol/00VguFa4663KFFR8kPGR/EvAnzs8e4GzLNxzjXY3LOCA
G+XsEpAqj9DVblfTcjjwbfswTBEK2KFdZ5NLSuNJwfoPCyymYqTAr7wlShpp
kg6ns+umoX/vwvSvPdL4mObTvSJFEeq8n1kdK7lIKh/tta1UQlC2Sl5TBfPP
kpacimpVhuUZ61caCjwk3U/d/Z2YAc2tVEBhOA+6gV8e/aWjgiUJuwwTn/Lg
HOJrX+89EyE5Wp3qZnwkmSTsUDg5C/FFW54/SuNj9o4KsuiKKi5br5itMsDH
Ys7A31Hi2ej4kjznwDIZlFHGirl8NZwgsvbey5JBeVjM3v/s1DEU/qvtmEQG
PU61wnYrDYxJxzistZ8C5eympMZmDVxNCdW1PTEFCkoefj3bNBF6ts+OejcF
NxoHI87+0ERJF8fFSF8WW3w39MYc1sKiCylWaUGy6NJa3RujrY2fnYp12tdl
Ma7hqKpzXRtvfF15W77Kosexqf+f1Tp4+jW73sRCDu1ucxXaB3RQYypb3R0u
h6CcxO0+kXMRGxOuW1QmBxEZtEV2ui7+sJxRYtgnh85761O9i3Rx+2Ckm9Qs
eQw7qak8M9PD0YZFO2atkEcTN9DEv00P5Un9XOW98lj9896hfF99BFidr+/M
l0euekbof//p432NfP9hkTwyosPSJ1IM8J4QfHQalUfmvXmUmto82Lij0FGR
hI9egLrvnXkoufBq5LQJiTbRZX+hoyFcI1TZwatIyOTtFq7qM4RD2YOkpm0k
XixcI7nZbwg2Lm7U9SdR0GB4Wn3QEB6xfWNxjG0+vRuQSAxxVD54u2UAiTBz
p5S0cUM0vp48lP8nia4W9ecv5WnYiLVcA4NJlP54vGmzKY0ROkHwJIxERPIF
2RZzGua8t1u1d5KwV4+5brqIRkG6lG8E4257Ey7XhsanrrhstV0kpp3ILSkW
0lj75tn8P8NJROqGSYY8aZjMifj7WwQJJ1fVmD1xNOZ+vv29O4oEd6Kz6lYC
jZMiJy/daBK1+Vk/xhJpTPcljwYzXjyqHB5+lMYM52n2E4z1TyoEhaXRsCi+
uEAplsQHyycllRk0htcLxR6M894d65OcohH7698t5xjPEMhu3p7D3C90b43B
fhJP/27MKz9L49Y6u9wQximxh7uH8mjkB8raX2PMfSLtGVRIY41PymerOKZn
d/2pS0U03N3xNZZxjFrC84ES5v53e2vrGC9++JvivMs0BKKYc1bxJL4Fs9YE
lNNo/rW/dGECiQrl+ynFFTQs37tlGBwgEVQd3dJ3lcY98vtGjUSmX25CuPUW
jRXHUtLlDzH91+8kFt6hUSWz5xdxmISX1566D1U0aI2ZDuOMn136ZrO5hsbV
7UGR4qNMn6PEzLeRhtmX+57VKUzfaPnOc800HLJaH1w9zvRlB1d2i2gMHAjg
F58gUdk/QHs/pXEpxdg0NZU57/BHbc9XzHqvt7RfJonoh11T1w7ReKOQeYRz
loSlXcOQcKoRCk6d/jKvjMQFdlfJLUUjdF57fkpQToKsG/bTm26ElelqyuZX
SLxfqtLNVTXCwblV2ahk3ndZYGuDjhFs529sd71B4qPD1CtLLY1QPrhaHFZN
ItXZe4ftRiPULO2MSRORGPT6NmpeZoTWp2s/eQ2RcLw2b0GzjTEGSnddYelR
qBxY/CheZIzx6hbCzpvC25Xtox7+8zE51SlgXwaF24NB6198n4+Q9NjQDY8o
RHJc9m+YmA9rez1H08cUrGdblLz7yTgjc1iukUJP4f7ocCkBdp/Jz65uorD1
ueOTWK4ATs9WxsxupbDD9G1YuoIAUo5v/2zroJA4yrtbrSPAcoVLxxV6mf0x
katYugIUGROiXsYOU4Z67fQEcD3T1nH9PYUpWk9lHxsIEL0ub6n7Rwon1uR4
PzUWoH6hOCfnHwrZFQKpPksBxvXyTVU/U9i35EKGwWIBNvbYKQ0w9nqsPC/I
SoCzbZeqb0soqL8bdxmxFuDL8dJgt1EKBVRDwYSdAPcCDH5L/sb0nbFYZGMv
ALegM9hjjOnTKxXtX8b0xo1v1v2XgoHt8TFpBwHEnE2X739n+pqlkhxWCEDl
p/5K+kFhcN0u9SRHASabebPcxymIesXXRE4CbIi4+Ut7gkJ5iIeDwkoBHDLS
Ln9m/D8UG0m4
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{2., 2.},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubsuperscriptBox[\\(\[CapitalDelta]m\\), \\(atm\\), \
\\(2\\)]\\) [\\!\\(\\*SuperscriptBox[\\(10\\), \\(-3\\)]\\)\[ThinSpace]\\!\\(\
\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)]\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{{{-10., 
       FormBox[
        TagBox[
         InterpretationBox["\"-10.0\"", -10., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-9.8\"", -9.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.600000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-9.6\"", -9.600000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-9.4\"", -9.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-9.2\"", -9.200000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-9., 
       FormBox[
        TagBox[
         InterpretationBox["\"-9.0\"", -9., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.8\"", -8.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.6\"", -8.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.4\"", -8.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-8.2\"", -8.200000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-8., 
       FormBox[
        TagBox[
         InterpretationBox["\"-8.0\"", -8., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-7.8\"", -7.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-7.6\"", -7.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-7.4\"", -7.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-7.2\"", -7.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-7., 
       FormBox[
        TagBox[
         InterpretationBox["\"-7.0\"", -7., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-6.8\"", -6.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-6.6\"", -6.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-6.4\"", -6.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-6.2\"", -6.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-6., 
       FormBox[
        TagBox[
         InterpretationBox["\"-6.0\"", -6., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-5.8\"", -5.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-5.6\"", -5.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-5.4\"", -5.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-5.2\"", -5.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-5., 
       FormBox[
        TagBox[
         InterpretationBox["\"-5.0\"", -5., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-4.8\"", -4.800000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-4.6\"", -4.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-4.4\"", -4.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-4.2\"", -4.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-4., 
       FormBox[
        TagBox[
         InterpretationBox["\"-4.0\"", -4., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-3.8\"", -3.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-3.6\"", -3.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-3.4\"", -3.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-3.2\"", -3.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-3., 
       FormBox[
        TagBox[
         InterpretationBox["\"-3.0\"", -3., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-2.8\"", -2.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-2.6\"", -2.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-2.4\"", -2.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-2.2\"", -2.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-2., 
       FormBox[
        TagBox[
         InterpretationBox["\"-2.0\"", -2., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-1.8\"", -1.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.6, 
       FormBox[
        TagBox[
         InterpretationBox["\"-1.6\"", -1.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.4000000000000001`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-1.4\"", -1.4000000000000001`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1.2000000000000002`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-1.2\"", -1.2000000000000002`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-1., 
       FormBox[
        TagBox[
         InterpretationBox["\"-1.0\"", -1., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.8, 
       FormBox[
        TagBox[
         InterpretationBox["\"-0.8\"", -0.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.6000000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\"-0.6\"", -0.6000000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.4, 
       FormBox[
        TagBox[
         InterpretationBox["\"-0.4\"", -0.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {-0.2, 
       FormBox[
        TagBox[
         InterpretationBox["\"-0.2\"", -0.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0., 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.0\"", 0., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.2\"", 0.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.4\"", 0.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.6000000000000001, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 0.6\"", 0.6000000000000001, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {0.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 0.8\"", 0.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1., 
       FormBox[
        TagBox[
         InterpretationBox["\" 1.0\"", 1., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.2000000000000002`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 1.2\"", 1.2000000000000002`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.4000000000000001`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 1.4\"", 1.4000000000000001`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 1.6\"", 1.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {1.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 1.8\"", 1.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2., 
       FormBox[
        TagBox[
         InterpretationBox["\" 2.0\"", 2., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 2.2\"", 2.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 2.4\"", 2.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 2.6\"", 2.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {2.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 2.8\"", 2.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3., 
       FormBox[
        TagBox[
         InterpretationBox["\" 3.0\"", 3., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 3.2\"", 3.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.4000000000000004`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 3.4\"", 3.4000000000000004`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 3.6\"", 3.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {3.8000000000000003`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 3.8\"", 3.8000000000000003`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4., 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.0\"", 4., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.2\"", 4.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.4\"", 4.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 4.6\"", 4.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {4.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 4.8\"", 4.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5., 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.0\"", 5., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.2\"", 5.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.4\"", 5.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 5.6\"", 5.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {5.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 5.8\"", 5.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6., 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.0\"", 6., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.2\"", 6.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.4\"", 6.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 6.6\"", 6.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {6.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 6.8\"", 6.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7., 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.0\"", 7., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.2, 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.2\"", 7.2, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.4\"", 7.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.6000000000000005`, 
       FormBox[
        TagBox[
         InterpretationBox[
         "\" 7.6\"", 7.6000000000000005`, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {7.800000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 7.8\"", 7.800000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8., 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.0\"", 8., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.2\"", 8.200000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.4\"", 8.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.6, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.6\"", 8.6, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {8.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 8.8\"", 8.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9., 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.0\"", 9., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.200000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.2\"", 9.200000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.4, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.4\"", 9.4, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.600000000000001, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.6\"", 9.600000000000001, AutoDelete -> True],
          PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {9.8, 
       FormBox[
        TagBox[
         InterpretationBox["\" 9.8\"", 9.8, AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}, {10., 
       FormBox[
        TagBox[
         InterpretationBox["\" 10.0\"", 10., AutoDelete -> True], 
         PaddedForm[#, {2, 1}]& ], TraditionalForm]}}, {{0., 
       FormBox["\"\"", TraditionalForm]}, {0.2, 
       FormBox["\"\"", TraditionalForm]}, {0.4, 
       FormBox["\"\"", TraditionalForm]}, {0.6000000000000001, 
       FormBox["\"\"", TraditionalForm]}, {0.8, 
       FormBox["\"\"", TraditionalForm]}, {1., 
       FormBox["\"\"", TraditionalForm]}, {1.2000000000000002`, 
       FormBox["\"\"", TraditionalForm]}, {1.4000000000000001`, 
       FormBox["\"\"", TraditionalForm]}, {1.6, 
       FormBox["\"\"", TraditionalForm]}, {1.8, 
       FormBox["\"\"", TraditionalForm]}, {2., 
       FormBox["\"\"", TraditionalForm]}, {2.2, 
       FormBox["\"\"", TraditionalForm]}, {2.4000000000000004`, 
       FormBox["\"\"", TraditionalForm]}, {2.6, 
       FormBox["\"\"", TraditionalForm]}, {2.8000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {3., 
       FormBox["\"\"", TraditionalForm]}, {3.2, 
       FormBox["\"\"", TraditionalForm]}, {3.4000000000000004`, 
       FormBox["\"\"", TraditionalForm]}, {3.6, 
       FormBox["\"\"", TraditionalForm]}, {3.8000000000000003`, 
       FormBox["\"\"", TraditionalForm]}, {4., 
       FormBox["\"\"", TraditionalForm]}, {4.2, 
       FormBox["\"\"", TraditionalForm]}, {4.4, 
       FormBox["\"\"", TraditionalForm]}, {4.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {4.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {5., 
       FormBox["\"\"", TraditionalForm]}, {5.2, 
       FormBox["\"\"", TraditionalForm]}, {5.4, 
       FormBox["\"\"", TraditionalForm]}, {5.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {5.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {6., 
       FormBox["\"\"", TraditionalForm]}, {6.2, 
       FormBox["\"\"", TraditionalForm]}, {6.4, 
       FormBox["\"\"", TraditionalForm]}, {6.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {6.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {7., 
       FormBox["\"\"", TraditionalForm]}, {7.2, 
       FormBox["\"\"", TraditionalForm]}, {7.4, 
       FormBox["\"\"", TraditionalForm]}, {7.6000000000000005`, 
       FormBox["\"\"", TraditionalForm]}, {7.800000000000001, 
       FormBox["\"\"", TraditionalForm]}, {8., 
       FormBox["\"\"", TraditionalForm]}, {8.200000000000001, 
       FormBox["\"\"", TraditionalForm]}, {8.4, 
       FormBox["\"\"", TraditionalForm]}, {8.6, 
       FormBox["\"\"", TraditionalForm]}, {8.8, 
       FormBox["\"\"", TraditionalForm]}, {9., 
       FormBox["\"\"", TraditionalForm]}, {9.200000000000001, 
       FormBox["\"\"", TraditionalForm]}, {9.4, 
       FormBox["\"\"", TraditionalForm]}, {9.600000000000001, 
       FormBox["\"\"", TraditionalForm]}, {9.8, 
       FormBox["\"\"", TraditionalForm]}, {10., 
       FormBox["\"\"", TraditionalForm]}}}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.63684712581448*^9, 3.636847467341304*^9}]
}, Open  ]],

Cell["RG change of the mass squared differences:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\
\), \(2\)], \(sol\)]\) is:     \!\(\*SubscriptBox[SuperscriptBox[\(\
\[CapitalDelta]m\), \(2\)], \(sol\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\
\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \(sol\)]\)(\!\
\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", ",", 
    RowBox[{
     RowBox[{"\[CapitalDelta]m2sol", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[CapitalDelta]m2sol", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", 
    "\"\< \!\(\*SuperscriptBox[\(eV\), \(2\)]\)\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<The RG change of \!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\
\), \(2\)], \(atm\)]\) is:     \!\(\*SubscriptBox[SuperscriptBox[\(\
\[CapitalDelta]m\), \(2\)], \(atm\)]\)(\!\(\*SubscriptBox[\(M\), \(GUT\)]\))-\
\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \(atm\)]\)(\!\
\(\*SubscriptBox[\(M\), \(Z\)]\)) = \>\"", ",", 
    RowBox[{
     RowBox[{"\[CapitalDelta]m2atm", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "GUT"}], "]"}], "]"}], "-", 
     RowBox[{"\[CapitalDelta]m2atm", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"10", ",", "MZ"}], "]"}], "]"}]}], ",", 
    "\"\< \!\(\*SuperscriptBox[\(eV\), \(2\)]\)\>\""}], "]"}], 
  ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[SuperscriptBox[\\(\
\[CapitalDelta]m\\), \\(2\\)], \\(sol\\)]\\) is:     \
\\!\\(\\*SubscriptBox[SuperscriptBox[\\(\[CapitalDelta]m\\), \\(2\\)], \\(sol\
\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[SuperscriptBox[\\(\[CapitalDelta]m\\), \
\\(2\\)], \\(sol\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", "0.00008281272704730184`", 
   "\[InvisibleSpace]", "\<\" \\!\\(\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)\"\
\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \
\(2\)], \(sol\)]\) is:     \
\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \(sol\)]\)(\!\
\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\*SubscriptBox[SuperscriptBox[\(\
\[CapitalDelta]m\), \(2\)], \(sol\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \
", 0.00008281272704730184, " \!\(\*SuperscriptBox[\(eV\), \(2\)]\)"],
  Editable->False]], "Print",
 CellChangeTimes->{3.63684712587848*^9, 3.6368474673944283`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The RG change of \\!\\(\\*SubscriptBox[SuperscriptBox[\\(\
\[CapitalDelta]m\\), \\(2\\)], \\(atm\\)]\\) is:     \
\\!\\(\\*SubscriptBox[SuperscriptBox[\\(\[CapitalDelta]m\\), \\(2\\)], \\(atm\
\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \
\\(GUT\\)]\\))-\\!\\(\\*SubscriptBox[SuperscriptBox[\\(\[CapitalDelta]m\\), \
\\(2\\)], \\(atm\\)]\\)(\\!\\(\\*SubscriptBox[\\(M\\), \\(Z\\)]\\)) = \"\>", 
   "\[InvisibleSpace]", "0.0019050199501923306`", 
   "\[InvisibleSpace]", "\<\" \\!\\(\\*SuperscriptBox[\\(eV\\), \\(2\\)]\\)\"\
\>"}],
  SequenceForm[
  "The RG change of \!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \
\(2\)], \(atm\)]\) is:     \
\!\(\*SubscriptBox[SuperscriptBox[\(\[CapitalDelta]m\), \(2\)], \(atm\)]\)(\!\
\(\*SubscriptBox[\(M\), \(GUT\)]\))-\!\(\*SubscriptBox[SuperscriptBox[\(\
\[CapitalDelta]m\), \(2\)], \(atm\)]\)(\!\(\*SubscriptBox[\(M\), \(Z\)]\)) = \
", 0.0019050199501923306`, " \!\(\*SuperscriptBox[\(eV\), \(2\)]\)"],
  Editable->False]], "Print",
 CellChangeTimes->{3.63684712587848*^9, 3.636847467400426*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CP asymmetry in leptogenesis ", "Section"],

Cell[TextData[{
 "In order to calculate the CP asymmetry, the Majorana mass matrix of the \
right-handed and the left-handed neutrinos and the neutrino Yukawa couplings \
are needed. The returned matrices are the matrices which are valid at the \
given scale pScale. RGEM\[Nu]r and RGEY\[Nu] can also be used, but the \
returned matrices also contain the values of the right-handed neutrinos which \
are already integrated out.\n\nIn the following, the CP asymmetry (MSSM) in \
thermal leptogenesis in the limit ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["M", "1"], 
     RowBox[{"<<", " ", 
      SubscriptBox["M", "2"]}]}], ",", 
    SubscriptBox["M", "3"]}], TraditionalForm]]],
 "is calculated. If you want to calculate the CP asymmetry in a different \
scenario, you have to change the formula appropiately."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Epsilon]", "[", "pScale_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"lM", ",", "lY", ",", "lm", ",", "lvev"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"right", "-", 
       RowBox[{"handed", " ", "neutrino", " ", "mass", " ", "matrix"}]}], " ",
       "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"lM", "=", 
       RowBox[{"RGEGetSolution", "[", 
        RowBox[{"pScale", ",", "RGERawM\[Nu]r", ",", "1"}], "]"}]}], ";", " ",
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"neutrino", " ", "Yukawa", " ", "couplings", " ", "matrix"}], 
       " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"lY", "=", 
       RowBox[{"RGEGetSolution", "[", 
        RowBox[{"pScale", ",", "RGERawY\[Nu]", ",", "1"}], "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"light", " ", "neutrino", " ", "mass", " ", "matrix"}], " ", 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"lm", "=", 
       RowBox[{
        RowBox[{"RGEGetSolution", "[", 
         RowBox[{"pScale", ",", "RGEM\[Nu]", ",", "1"}], "]"}], "*", 
        RowBox[{"10", "^", 
         RowBox[{"-", "9"}]}]}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{"change", " ", "to", " ", "the", " ", "basis"}], ",", " ", 
        RowBox[{"where", " ", "M", " ", "is", " ", "diagonal"}]}], " ", 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"lM", ",", "lY"}], "}"}], "=", 
       RowBox[{"RGERotateM", "[", 
        RowBox[{"lM", ",", "lY"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{
        "the", " ", "mass", " ", "of", " ", "the", " ", "lightest", " ", 
         "right"}], "-", 
        RowBox[{"handed", " ", "neutrino"}]}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"lM1", "=", 
       RowBox[{"lM", "\[LeftDoubleBracket]", 
        RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{
        "the", " ", "Yukawa", " ", "couplings", " ", "of", " ", "the", " ", 
         "lightest", " ", "right"}], "-", 
        RowBox[{"handed", " ", "neutrino"}]}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"lY1", "=", 
       RowBox[{"lY", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"lvev", "=", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"N", "[", 
             RowBox[{"RGEvEW", "*", 
              RowBox[{"RGEtan\[Beta]", "/", 
               RowBox[{"Sqrt", "[", 
                RowBox[{"1", "+", 
                 RowBox[{"RGEtan\[Beta]", "^", "2"}]}], "]"}]}]}], "]"}], "/", 
            RowBox[{"Sqrt", "[", "2", "]"}]}], ")"}], "/.", 
          RowBox[{
           RowBox[{"RGEGetOptions", "[", "\"\<MSSM\>\"", "]"}], 
           "\[LeftDoubleBracket]", 
           RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}]}], ")"}], "/.", 
        
        RowBox[{
         RowBox[{"RGEGetModelOptions", "[", "\"\<MSSM\>\"", "]"}], 
         "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"CP", " ", "asymmetry"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"leps", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"3", "/", "8"}], "/", "\[Pi]"}], "*", 
        RowBox[{"lM1", "/", 
         RowBox[{"lvev", "^", "2"}]}], "*", 
        RowBox[{
         RowBox[{"Im", "[", 
          RowBox[{"(", 
           RowBox[{"lY1", ".", 
            RowBox[{"Conjugate", "[", "lm", "]"}], ".", "lY1"}], ")"}], "]"}],
          "/", 
         RowBox[{"(", 
          RowBox[{"lY1", ".", 
           RowBox[{"Conjugate", "[", "lY1", "]"}]}], ")"}]}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "leps", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RGEGetTransitions", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Infinity]", ",", "\<\"MSSM\"\>", ",", 
     RowBox[{"{", 
      RowBox[{"RGEtan\[Beta]", "\[Rule]", "50"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2.162625661103112`*^14", ",", "\<\"MSSM\"\>", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"RGEAutoGenerated", "\[Rule]", "True"}], ",", 
       RowBox[{"RGEIntegratedOut", "\[Rule]", "1"}], ",", 
       RowBox[{"RGEtan\[Beta]", "\[Rule]", "50"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.6691461405955562`*^12", ",", "\<\"MSSM\"\>", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"RGEAutoGenerated", "\[Rule]", "True"}], ",", 
       RowBox[{"RGEIntegratedOut", "\[Rule]", "2"}], ",", 
       RowBox[{"RGEtan\[Beta]", "\[Rule]", "50"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3.6429522028436714`*^10", ",", "\<\"MSSM0N\"\>", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"RGEAutoGenerated", "\[Rule]", "True"}], ",", 
       RowBox[{"RGEtan\[Beta]", "\[Rule]", "50"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1000", ",", "\<\"SM0N\"\>", ",", 
     RowBox[{"{", 
      RowBox[{"RGEAutoGenerated", "\[Rule]", "\<\"SM\"\>"}], "}"}]}], "}"}]}],
   "}"}]], "Output",
 CellChangeTimes->{3.636847125947072*^9, 3.636847467437077*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Epsilon]", "[", 
  RowBox[{
   RowBox[{"RGEGetTransitions", "[", "]"}], "\[LeftDoubleBracket]", 
   RowBox[{"4", ",", "1"}], "\[RightDoubleBracket]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"3.380652680148336`*^-10", "+", 
  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.636847125978827*^9, 3.63684746746513*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"However", ",", 
  RowBox[{
  "the", " ", "CP", " ", "asymmetry", " ", "in", " ", "thermal", " ", 
   "leptogenesis", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"in", " ", "the", " ", "limit", " ", 
      SubscriptBox["M", 
       RowBox[{"1", " "}]], 
      RowBox[{"<<", " ", 
       SubscriptBox["M", "2"]}]}], ",", 
     SubscriptBox["M", "3"]}], ")"}], " ", "is", " ", "much", " ", "easier", 
   " ", "obtained", " ", "directly", " ", "by", " ", 
   RowBox[{"RGEGetSolution", "."}]}]}]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RGEGetSolution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"RGEGetTransitions", "[", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{"4", ",", "1"}], "\[RightDoubleBracket]"}], ",", "RGE\[Epsilon]1",
    ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"3.380652680148336`*^-10", "+", 
  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.636847126009606*^9, 3.6368474674930067`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"RGEGetSolution", "[", 
      RowBox[{
       RowBox[{"10", "^", "t"}], ",", "RGE\[Epsilon]1"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"Log", "[", 
        RowBox[{"10", ",", 
         RowBox[{
          RowBox[{"RGEGetTransitions", "[", "]"}], "\[LeftDoubleBracket]", 
          RowBox[{"4", ",", "1"}], "\[RightDoubleBracket]"}]}], "]"}], ",", 
       "tmax"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(\[Epsilon]\), \(1\)]\)\>\""}], "}"}]}], ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", "Automatic", ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", "Automatic"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", "Blue", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.435655668026875*^9, 3.435655697375855*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV03kw3GcYB/BlSRzjqiOkmYhj79/Pj0glW956JpLNKMFolthKR9WRjWs1
hBjUEWdItqNT6WTk6oSENMjWtaJESqipK9EsIxpJuipYRiWvok3f/vHOO5//
nuf9fl+n6JTQWH0Wi3WInP/vP1+HaM44coCve1wT5I2RWFyV84LmAHb2jmzw
wai8ZMI5yIcD/b2zLoYfYUS7ypI4ERxQgWa4ETDicHj2d6s5oC1W+AVLMFqv
WX5lNc6Bq9s6NiMDMPK7HZAVYsqFq2tg4xyC0cHsBI7cnwtab127gxSjXY+K
VyMLuRAlU3WVR2BUPxk7IezlgtnMvlrXzzBqytXkPX7HBc1lPasvozFSXpYE
hYt5wPYMlRyKw8g1L9OjJYMHm0/+/djzFEZV0ze9dM08aNjo+P1OEkZZQ3EG
+gs8OOLUX/iHAiPbgoW5ZT4fOG33jJk0sl9TwlJ7NB+6KZcPzDMwUtU2GkVe
58Mz/eFU6yyMpEzJ4uQMH8TXkw2eZWM0xshW99oLoI43YHP4K4z01lfM4sIE
wJhMeTbkY2R6oDAgTSkAa9a5nJZCjAZ9J69EjgjgmIP0gaoII9ZUr9rRTAh9
DwPFihKMrBbr1rskQnj/RB+aLsVo7Xi280asEL6ui3GoLMPot76QQK8iIVzz
lZUy5RjV6py3R90QApfu5KqJz+x486CsWwij3G2CA+cxKjC2mt0zLYQUZKao
I76wSbHa14UADVKlSQVGaWNC4f0dIji1PzT8JLGsjn+sx0sEjH/iozZiWuxq
ce8TEVTk1qyuEbOGHH/5XiGCTJVNFacSox/gn4v6ShFsLCbYBRLLWqeknzeK
YBDpRr4g9nujl0cPi0D34kZTIjG1T1D/96IIbjEpB+OJ9ZmgWJUxBTtneBIp
8aTg9J4kLgVz7PtDXsTlRlnaqsMUGLTt/2478YdzeXfUMRQ4bmlrB8k8mvwi
9kg+BYaGT7MLiNN3lcteXqHAUuqQ4EYcnKexje2koJR9W7RK9he84o5pn1LA
+3a+q494WhRePfGWArlRp7yUWHm65MTPtjScS7SKR8QeGS0/XttLQ5OLsfkC
ed/xrJemOcE0aAu4PRXER6P/SupPpuGJWeeWPfGAP3vU4gINyS2wu5rk5b5k
+VDSQIPCQ2tnRXxJubs1Z4CGOOvm1lyS76z815h+PTfIKHLv8Sb5z5iAYN7X
Dczl6rA60o8Wl7sWc2fdYOzS+MQG6Q8YPDftbneDSpvjBaukX34ZN0dT37rB
Mh3Zpib9c4qOa3vHMKCLOD+fnkP6a7Ogik9mwCbK3nyA9NfRN2G8/hYDR2LV
g+9lYsTeyu0YmGVgfN5OLE4n+4Do0+cW7iAPK5PPp2JU/NM3rwcpd5ioPcmt
SSb5W6/sXPJxhxWv9pzGRIzOxgcEWB51B3ajuKmZ/Lf/AGCj9ig=
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{11., 3.38*^-10},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(\[Epsilon]\\), \\(1\\)]\\)\"", 
      TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{Automatic, Automatic}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.636847126177956*^9, 3.636847467659504*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Effective", " ", "neutrino", " ", "mass", 
  "\"\<\!\(\*SubscriptBox[OverscriptBox[\(m\), \(~\)], \(1\)]\)\>\"", 
  " "}]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"RGEGetSolution", "[", 
      RowBox[{
       RowBox[{"10", "^", "t"}], ",", "RGEM1Tilde"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"Log", "[", 
        RowBox[{"10", ",", 
         RowBox[{
          RowBox[{"RGEGetTransitions", "[", "]"}], "\[LeftDoubleBracket]", 
          RowBox[{"4", ",", "1"}], "\[RightDoubleBracket]"}]}], "]"}], ",", 
       "tmax"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
     RowBox[{"FrameLabel", "->", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(log\), \(10\)]\)(\[Mu]/GeV)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[OverscriptBox[\(m\), \(~\)], \(1\)]\) \
[eV]\>\""}], "}"}]}], ",", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"RGELogTicksLabeled", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", "Automatic", ",", 
        RowBox[{"RGELogTicks", "[", 
         RowBox[{"2", ",", "19"}], "]"}], ",", "Automatic"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"All", ",", "All"}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", "Blue", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Prolog", "\[Rule]", 
      RowBox[{"{", "ShadowEFT", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], 
    "\[IndentingNewLine]", "]"}], ",", 
   RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.43565567109143*^9, 3.435655699340998*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], 
    LineBox[CompressedData["
1:eJwV0Hk41fkXB/BrSRpCYrpKmeJy7/VxfUmG+dHnyFJCaaGx/cIgg4xibJO9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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{11., 0.020020000000000003`},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"\\!\\(\\*SubscriptBox[OverscriptBox[\\(m\\), \\(~\\)], \\(1\\)]\\) \
[eV]\"", TraditionalForm], None}, {
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(log\\), \\(10\\)]\\)(\[Mu]/GeV)\"", 
      TraditionalForm], None}},
  FrameTicks->{{Automatic, Automatic}, {{{2, 
       FormBox["2", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["3", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["4", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["5", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["6", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["7", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["8", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["9", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["10", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["11", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["12", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["13", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["14", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["15", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["16", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["17", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["18", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["19", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}, {{2, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200], 2.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300], 2.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400], 2.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500], 2.6989700043360183`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600], 2.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700], 2.8450980400142565`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800], 2.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900], 2.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {3, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000], 3.301029995663981], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000], 3.4771212547196617`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000], 3.6020599913279616`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000], 3.6989700043360187`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000], 3.7781512503836425`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000], 3.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000], 3.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000], 3.9542425094393243`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {4, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000], 4.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000], 4.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000], 4.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000], 4.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000], 4.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000], 4.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000], 4.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000], 4.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {5, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000], 5.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000], 5.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000], 5.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000], 5.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000], 5.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000], 5.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000], 5.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000], 5.954242509439324], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {6, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000], 6.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000], 6.477121254719662], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000], 6.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000], 6.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000], 6.778151250383643], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000], 6.845098040014256], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000], 6.903089986991943], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000], 6.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {7, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000], 7.3010299956639795`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000], 7.477121254719661], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000], 7.602059991327962], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000], 7.698970004336018], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000], 7.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000], 7.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000], 7.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000], 7.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {8, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000], 8.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000], 8.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000], 8.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000], 8.698970004336019], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000], 8.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000], 8.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000], 8.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000], 8.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {9, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000], 9.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000], 9.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000], 9.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000], 9.698970004336017], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000], 9.778151250383642], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000], 9.845098040014255], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000], 9.903089986991942], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000], 9.954242509439323], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {10, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000], 10.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000], 10.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000], 10.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000], 10.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000], 10.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000], 10.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000], 10.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000], 10.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {11, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000], 11.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000], 11.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000], 11.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000], 11.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000], 11.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000], 11.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000], 11.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000], 11.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {12, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000], 12.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000], 12.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000], 12.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000], 12.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000], 12.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000], 12.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000], 12.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000], 12.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {13, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000], 13.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000], 13.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000], 13.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000], 13.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000], 13.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000], 13.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000], 13.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000], 13.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {14, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000], 14.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000], 14.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000], 14.602059991327959`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000], 14.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000], 14.778151250383642`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000], 14.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000], 14.903089986991942`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000], 14.954242509439322`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {15, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000], 15.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000], 15.477121254719659`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000], 15.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000], 15.698970004336017`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000], 15.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000], 15.845098040014255`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000], 15.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000], 15.954242509439323`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {16, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000], 16.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000], 16.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000], 16.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000], 16.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000], 16.778151250383644`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000], 16.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000], 16.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000], 16.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {17, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[200000000000000000], 17.30102999566398], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[300000000000000000], 17.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[400000000000000000], 17.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[500000000000000000], 17.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[600000000000000000], 17.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[700000000000000000], 17.845098040014257`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[800000000000000000], 17.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[900000000000000000], 17.954242509439325`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {18, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[2000000000000000000], 18.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[3000000000000000000], 18.477121254719663`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[4000000000000000000], 18.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[5000000000000000000], 18.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[6000000000000000000], 18.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[7000000000000000000], 18.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[8000000000000000000], 18.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[9000000000000000000], 18.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {19, 
       FormBox["\"\"", TraditionalForm]}, {
       NCache[Log[10]^(-1) Log[20000000000000000000], 19.301029995663978`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[30000000000000000000], 19.47712125471966], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[40000000000000000000], 19.60205999132796], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[50000000000000000000], 19.698970004336015`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[60000000000000000000], 19.77815125038364], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[70000000000000000000], 19.845098040014253`], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[80000000000000000000], 19.90308998699194], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}, {
       NCache[Log[10]^(-1) Log[90000000000000000000], 19.95424250943932], 
       FormBox["\"\"", TraditionalForm], {0.00375, 0}}}}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImageSize->400,
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotLabel->FormBox["\"\"", TraditionalForm],
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Prolog->{{{
      GrayLevel[0.9], 
      RectangleBox[{10.561453473142157`, -10000}, {
       12.22249436260837, 10000}]}, {
      GrayLevel[0.8], 
      RectangleBox[{12.22249436260837, -10000}, {
       14.334981351900138`, 10000}]}, {
      GrayLevel[0.7], 
      RectangleBox[{14.334981351900138`, -10000}, {17, 10000}]}}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.6368471263493433`*^9, 3.636847467833156*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1916, 1041},
WindowMargins->{{0, Automatic}, {Automatic, 18}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
ShowSelection->True,
FrontEndVersion->"10.0 for Linux x86 (64-bit) (December 4, 2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 42, 0, 90, "Title"],
Cell[612, 24, 157, 3, 31, "Text"],
Cell[772, 29, 699, 17, 121, "Input"],
Cell[CellGroupData[{
Cell[1496, 50, 190, 5, 55, "Input"],
Cell[CellGroupData[{
Cell[1711, 59, 145, 2, 43, "Print"],
Cell[1859, 63, 184, 3, 23, "Print"],
Cell[2046, 68, 203, 3, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[2276, 75, 262, 7, 55, "Input"],
Cell[2541, 84, 153, 3, 31, "Text"],
Cell[2697, 89, 74, 2, 32, "Input"],
Cell[CellGroupData[{
Cell[2796, 95, 42, 0, 64, "Section"],
Cell[2841, 97, 150, 3, 31, "Text"],
Cell[2994, 102, 121, 2, 32, "Input"],
Cell[3118, 106, 445, 11, 55, "Input"],
Cell[3566, 119, 488, 12, 52, "Text"],
Cell[4057, 133, 4220, 108, 561, "Input"],
Cell[8280, 243, 106, 3, 32, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8423, 251, 24, 0, 64, "Section"],
Cell[CellGroupData[{
Cell[8472, 255, 33, 0, 43, "Subsection"],
Cell[8508, 257, 38, 0, 31, "Text"],
Cell[8549, 259, 370, 11, 77, "Input"],
Cell[8922, 272, 59, 0, 31, "Text"],
Cell[8984, 274, 4307, 121, 517, "Input"],
Cell[13294, 397, 158, 4, 51, "Text"],
Cell[13455, 403, 5133, 137, 297, "Input"],
Cell[18591, 542, 43, 0, 30, "Text"],
Cell[18637, 544, 608, 17, 81, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19282, 566, 60, 0, 44, "Subsection"],
Cell[CellGroupData[{
Cell[19367, 570, 62, 1, 32, "Input"],
Cell[19432, 573, 675, 14, 23, "Print"]
}, Open  ]],
Cell[20122, 590, 45, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[20192, 594, 2669, 74, 196, "Input"],
Cell[CellGroupData[{
Cell[22886, 672, 477, 10, 23, "Print"],
Cell[23366, 684, 482, 10, 23, "Print"],
Cell[23851, 696, 476, 10, 23, "Print"],
Cell[24330, 708, 617, 12, 23, "Print"],
Cell[24950, 722, 623, 12, 23, "Print"],
Cell[25576, 736, 456, 10, 23, "Print"],
Cell[26035, 748, 456, 10, 23, "Print"],
Cell[26494, 760, 453, 10, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[27008, 777, 35, 0, 44, "Subsection"],
Cell[27046, 779, 51, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[27122, 783, 4124, 106, 253, "Input"],
Cell[31249, 891, 52005, 880, 299, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83291, 1776, 49, 0, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[83365, 1780, 1802, 44, 77, "Input"],
Cell[CellGroupData[{
Cell[85192, 1828, 835, 16, 23, "Print"],
Cell[86030, 1846, 837, 16, 23, "Print"],
Cell[86870, 1864, 833, 16, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[87776, 1888, 28, 0, 43, "Subsection"],
Cell[87807, 1890, 44, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[87876, 1894, 3977, 103, 253, "Input"],
Cell[91856, 1999, 50620, 857, 295, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[142513, 2861, 49, 0, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[142587, 2865, 1756, 43, 77, "Input"],
Cell[CellGroupData[{
Cell[144368, 2912, 577, 12, 23, "Print"],
Cell[144948, 2926, 847, 17, 23, "Print"],
Cell[145798, 2945, 854, 17, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[146725, 2970, 28, 0, 43, "Subsection"],
Cell[CellGroupData[{
Cell[146778, 2974, 41, 0, 35, "Subsubsection"],
Cell[146822, 2976, 54, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[146901, 2980, 4343, 114, 297, "Input"],
Cell[151247, 3096, 61751, 1116, 292, "Output"]
}, Open  ]],
Cell[213013, 4215, 40, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[213078, 4219, 1699, 45, 77, "Input"],
Cell[CellGroupData[{
Cell[214802, 4268, 757, 15, 23, "Print"],
Cell[215562, 4285, 757, 15, 23, "Print"],
Cell[216322, 4302, 758, 15, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[217141, 4324, 49, 0, 35, "Subsubsection"],
Cell[217193, 4326, 95, 2, 31, "Text"],
Cell[CellGroupData[{
Cell[217313, 4332, 3695, 96, 258, "Input"],
Cell[221011, 4430, 60082, 1044, 287, "Output"]
}, Open  ]],
Cell[281108, 5477, 95, 2, 31, "Text"],
Cell[CellGroupData[{
Cell[281228, 5483, 2090, 53, 178, "Input"],
Cell[283321, 5538, 64116, 1213, 297, "Output"]
}, Open  ]],
Cell[347452, 6754, 103, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[347580, 6761, 2306, 59, 200, "Input"],
Cell[349889, 6822, 62919, 1195, 293, "Output"]
}, Open  ]],
Cell[412823, 8020, 58, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[412906, 8024, 1527, 34, 65, "Input"],
Cell[CellGroupData[{
Cell[414458, 8062, 1096, 19, 23, "Print"],
Cell[415557, 8083, 1093, 19, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[416735, 8111, 48, 0, 65, "Section"],
Cell[416786, 8113, 854, 18, 91, "Text"],
Cell[417643, 8133, 4401, 109, 583, "Input"],
Cell[CellGroupData[{
Cell[422069, 8246, 65, 1, 32, "Input"],
Cell[422137, 8249, 1329, 32, 55, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[423503, 8286, 194, 4, 32, "Input"],
Cell[423700, 8292, 172, 3, 32, "Output"]
}, Open  ]],
Cell[423887, 8298, 529, 14, 31, "Text"],
Cell[CellGroupData[{
Cell[424441, 8316, 251, 6, 32, "Input"],
Cell[424695, 8324, 175, 3, 32, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[424907, 8332, 1703, 41, 143, "Input"],
Cell[426613, 8375, 40642, 678, 269, "Output"]
}, Open  ]],
Cell[467270, 9056, 155, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[467450, 9063, 1741, 42, 178, "Input"],
Cell[469194, 9107, 41570, 693, 287, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
